#!/usr/bin/env python

""" MultiQC submodule to parse output from Bcftools stats """

import logging
from collections import OrderedDict
from multiqc import config
from multiqc.plots import bargraph, linegraph, table

# Initialise the logger
log = logging.getLogger(__name__)


class StatsReportMixin:
    """Mixin loaded by the bcftools MultiqcModule class"""

    def parse_bcftools_stats(self):
        """
        Find bcftools stats logs and parse their data
          Bcftools stats reports contain 'sets' of data, which can
          have multiple vcf files each (but usually don't). Here,
          we treat each 'set' as a MultiQC sample, taking the first
          input filename for each set as the name.
        """
        collapse_complementary = getattr(config, "bcftools", {}).get("collapse_complementary_changes", False)
        if collapse_complementary:
            types = ["A>C", "A>G", "A>T", "C>A", "C>G", "C>T"]
        else:
            types = ["A>C", "A>G", "A>T", "C>A", "C>G", "C>T", "G>A", "G>C", "G>T", "T>A", "T>C", "T>G"]

        self.bcftools_stats = dict()
        self.bcftools_stats_indels = dict()
        self.bcftools_stats_vqc_snp = dict()
        self.bcftools_stats_sample_variants = dict()
        self.bcftools_stats_sample_tstv = dict()
        self.bcftools_stats_sample_singletons = dict()
        self.bcftools_stats_sample_depth = dict()
        self.bcftools_stats_vqc_transi = dict()
        self.bcftools_stats_vqc_transv = dict()
        self.bcftools_stats_vqc_indels = dict()
        depth_data = dict()
        for f in self.find_log_files("bcftools/stats"):
            s_names = list()
            for line in f["f"].splitlines():
                s = line.split("\t")
                # Get the sample names - one per 'set'
                if s[0] == "ID":
                    s_name = self.clean_s_name(s[2], f)
                    s_names.append(s_name)
                    if s_name in self.bcftools_stats:
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                    self.add_data_source(f, s_name, section="stats")
                    self.bcftools_stats[s_name] = dict()
                    self.bcftools_stats_indels[s_name] = dict()
                    self.bcftools_stats_sample_variants[s_name] = dict()
                    self.bcftools_stats_sample_tstv[s_name] = dict()
                    self.bcftools_stats_sample_singletons[s_name] = dict()
                    self.bcftools_stats_sample_depth[s_name] = dict()
                    self.bcftools_stats_vqc_snp[s_name] = dict()
                    self.bcftools_stats_vqc_transi[s_name] = dict()
                    self.bcftools_stats_vqc_transv[s_name] = dict()
                    self.bcftools_stats_vqc_indels[s_name] = dict()
                    depth_data[s_name] = OrderedDict()
                    self.bcftools_stats_indels[s_name][0] = None  # Avoid joining line across missing 0

                # Parse key stats
                if s[0] == "SN" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    field = s[2].strip()[:-1]
                    field = field.replace(" ", "_")
                    value = float(s[3].strip())
                    self.bcftools_stats[s_name][field] = value

                # Parse transitions/transversions stats
                if s[0] == "TSTV" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    fields = ["ts", "tv", "tstv", "ts_1st_ALT", "tv_1st_ALT", "tstv_1st_ALT"]
                    for i, f in enumerate(fields):
                        value = float(s[i + 2].strip())

                        self.bcftools_stats[s_name][f] = value

                # Parse substitution types
                if s[0] == "ST" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]

                    rc = {"A": "T", "C": "G", "G": "C", "T": "A"}
                    change = s[2].strip()
                    if change not in types:
                        change = ">".join(rc[n] for n in change.split(">"))

                    field = "substitution_type_{}".format(change)
                    value = float(s[3].strip())
                    if field not in self.bcftools_stats[s_name]:
                        self.bcftools_stats[s_name][field] = 0
                    self.bcftools_stats[s_name][field] += value

                # Indel length distributions
                if s[0] == "IDD" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    length = float(s[2].strip())
                    count = float(s[3].strip())
                    self.bcftools_stats_indels[s_name][length] = count

                # Per-sample counts
                if s[0] == "PSC" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    fields = ["variations_hom", "variations_het"]
                    for i, f in enumerate(fields):
                        self.bcftools_stats[s_name][f] = int(s[i + 4].strip())

                # Per-sample variant stats
                if s[0] == "PSC" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    sample = self.clean_s_name(s[2].strip(), f)
                    self.bcftools_stats_sample_variants[s_name][sample] = dict()
                    self.bcftools_stats_sample_variants[s_name][sample]["nSNPs"] = (
                        int(s[3].strip()) + int(s[4].strip()) + int(s[5].strip())
                    )
                    self.bcftools_stats_sample_variants[s_name][sample]["nIndels"] = int(s[8].strip())
                    nMissing = int(s[13].strip())
                    nPresent = self.bcftools_stats[s_name]["number_of_records"] - nMissing
                    self.bcftools_stats_sample_variants[s_name][sample]["nOther"] = (
                        nPresent
                        - self.bcftools_stats_sample_variants[s_name][sample]["nSNPs"]
                        - self.bcftools_stats_sample_variants[s_name][sample]["nIndels"]
                    )

                # Per-sample ts/tv stats
                if s[0] == "PSC" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    sample = self.clean_s_name(s[2].strip(), f)
                    self.bcftools_stats_sample_tstv[s_name][sample] = dict()
                    if int(s[7].strip()) != 0:
                        self.bcftools_stats_sample_tstv[s_name][sample]["tstv"] = float(
                            int(s[6].strip()) / int(s[7].strip())
                        )
                    else:
                        self.bcftools_stats_sample_tstv[s_name][sample]["tstv"] = 0

                # Per-sample singletons stats
                if s[0] == "PSC" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    sample = self.clean_s_name(s[2].strip(), f)
                    self.bcftools_stats_sample_singletons[s_name][sample] = dict()
                    self.bcftools_stats_sample_singletons[s_name][sample]["singletons"] = int(s[10].strip())
                    self.bcftools_stats_sample_singletons[s_name][sample]["rest"] = (
                        int(s[3].strip()) + int(s[4].strip()) + int(s[5].strip()) - int(s[10].strip())
                    )

                # Per-sample coverage stats
                if s[0] == "PSC" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    sample = self.clean_s_name(s[2].strip(), f)
                    self.bcftools_stats_sample_depth[s_name][sample] = dict()
                    self.bcftools_stats_sample_depth[s_name][sample]["depth"] = float(s[9].strip())

                # Depth plots
                if s[0] == "DP" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    bin_name = s[2].strip()
                    percent_sites = float(s[-1].strip())
                    depth_data[s_name][bin_name] = percent_sites

                # Variant Qualities
                if s[0] == "QUAL" and len(s_names) > 0:
                    s_name = s_names[int(s[1])]
                    quality = float("0" + s[2].strip())
                    self.bcftools_stats_vqc_snp[s_name][quality] = float(s[3].strip())
                    self.bcftools_stats_vqc_transi[s_name][quality] = float(s[4].strip())
                    self.bcftools_stats_vqc_transv[s_name][quality] = float(s[5].strip())
                    self.bcftools_stats_vqc_indels[s_name][quality] = float(s[6].strip())

        # Remove empty samples
        self.bcftools_stats = {k: v for k, v in self.bcftools_stats.items() if len(v) > 0}
        self.bcftools_stats_indels = {k: v for k, v in self.bcftools_stats_indels.items() if len(v) > 0}
        self.bcftools_stats_sample_variants = {
            k: v for k, v in self.bcftools_stats_sample_variants.items() if len(v) > 0
        }
        self.bcftools_stats_sample_tstv = {k: v for k, v in self.bcftools_stats_sample_tstv.items() if len(v) > 0}
        self.bcftools_stats_sample_singletons = {
            k: v for k, v in self.bcftools_stats_sample_singletons.items() if len(v) > 0
        }
        self.bcftools_stats_sample_depth = {k: v for k, v in self.bcftools_stats_sample_depth.items() if len(v) > 0}
        self.bcftools_stats_vqc_snp = {k: v for k, v in self.bcftools_stats_vqc_snp.items() if len(v) > 0}
        self.bcftools_stats_vqc_transi = {k: v for k, v in self.bcftools_stats_vqc_transi.items() if len(v) > 0}
        self.bcftools_stats_vqc_transv = {k: v for k, v in self.bcftools_stats_vqc_transv.items() if len(v) > 0}
        self.bcftools_stats_vqc_indels = {k: v for k, v in self.bcftools_stats_vqc_indels.items() if len(v) > 0}

        # Filter to strip out ignored sample names
        self.bcftools_stats = self.ignore_samples(self.bcftools_stats)

        if len(self.bcftools_stats) > 0:

            # Write parsed report data to a file
            self.write_data_file(self.bcftools_stats, "multiqc_bcftools_stats")

            # Stats Table
            stats_headers = self.bcftools_stats_genstats_headers()
            if getattr(config, "bcftools", {}).get("write_general_stats", True):
                self.general_stats_addcols(self.bcftools_stats, stats_headers, "Bcftools Stats")
            if getattr(config, "bcftools", {}).get("write_separate_table", False):
                self.add_section(
                    name="Bcftools Stats",
                    anchor="bcftools-stats_stats",
                    plot=table.plot(self.bcftools_stats, stats_headers),
                )

            # Make bargraph plot of substitution types
            keys = OrderedDict()
            for t in types:
                keys["substitution_type_{}".format(t)] = {"name": t}
            pconfig = {
                "id": "bcftools-stats-subtypes",
                "title": "Bcftools Stats: Substitutions",
                "ylab": "# Substitutions",
                "cpswitch_counts_label": "Number of Substitutions",
            }
            self.add_section(
                name="Variant Substitution Types",
                anchor="bcftools-stats_variant_sub_types",
                plot=bargraph.plot(self.bcftools_stats, keys, pconfig),
            )

            # Make histograms of variant quality
            if len(self.bcftools_stats_vqc_snp) > 0:
                pconfig = {
                    "id": "bcftools_stats_vqc",
                    "title": "Bcftools Stats: Variant Quality Count",
                    "ylab": "Count",
                    "xlab": "Quality",
                    "xDecimals": False,
                    "ymin": 0,
                    "smooth_points": 600,
                    # 'tt_label': '<b>{point.x} bp trimmed</b>: {point.y:.0f}',
                    "data_labels": [
                        {"name": "Count SNP", "ylab": "Quality"},
                        {"name": "Count Transitions", "ylab": "Quality"},
                        {"name": "Count Transversions", "ylab": "Quality"},
                        {"name": "Count Indels", "ylab": "Quality"},
                    ],
                }
                self.add_section(
                    name="Variant Quality",
                    anchor="bcftools-stats_variant_quality_plot",
                    plot=linegraph.plot(
                        [
                            self.bcftools_stats_vqc_snp,
                            self.bcftools_stats_vqc_transi,
                            self.bcftools_stats_vqc_transv,
                            self.bcftools_stats_vqc_indels,
                        ],
                        pconfig,
                    ),
                )

            # Make line graph of indel lengths
            if len(self.bcftools_stats_indels) > 0:
                pconfig = {
                    "id": "bcftools_stats_indel-lengths",
                    "title": "Bcftools Stats: Indel Distribution",
                    "ylab": "Count",
                    "xlab": "InDel Length (bp)",
                    "xDecimals": False,
                    "ymin": 0,
                }
                self.add_section(
                    name="Indel Distribution",
                    anchor="bcftools-stats_indel_plot",
                    plot=linegraph.plot(self.bcftools_stats_indels, pconfig),
                )
            # Make line graph of variants per depth
            if len(depth_data) > 0:
                pconfig = {
                    "id": "bcftools_stats_depth",
                    "title": "Bcftools Stats: Variant depths",
                    "ylab": "Fraction of sites (%)",
                    "xlab": "Variant depth",
                    "ymin": 0,
                    "ymax": 100,
                    "categories": True,
                }
                self.add_section(
                    name="Variant depths",
                    anchor="bcftools-stats_depth_plot",
                    description="Read depth support distribution for called variants",
                    plot=linegraph.plot(depth_data, pconfig),
                )

            # Make bargraph plot of missing sites
            if len(self.bcftools_stats_sample_variants) > 0:
                pconfig = {
                    "id": "bcftools-stats-sites",
                    "title": "Bcftools Stats: Sites per sample",
                    "ylab": "# Sites",
                    "cpswitch_counts_label": "Number of sites",
                    "data_labels": list(self.bcftools_stats_sample_variants),
                }
                self.add_section(
                    name="Sites per sample",
                    anchor="bcftools-stats_sites_per_sample",
                    plot=bargraph.plot(
                        list(self.bcftools_stats_sample_variants.values()), ["nSNPs", "nIndels", "nOther"], pconfig
                    ),
                )

            # Make bargraph plot of ts/tv stats
            if len(self.bcftools_stats_sample_tstv) > 0:
                pconfig = {
                    "id": "bcftools-stats-tstv",
                    "title": "Bcftools Stats: Ts/Tv",
                    "ylab": "Ts/Tv",
                    "cpswitch_counts_label": "Ts/Tv",
                    "cpswitch": False,
                    "data_labels": list(self.bcftools_stats_sample_tstv),
                }
                self.add_section(
                    name="Ts/Tv",
                    anchor="bcftools-stats_ts_tv",
                    plot=bargraph.plot(list(self.bcftools_stats_sample_tstv.values()), ["tstv"], pconfig),
                )

            # Make bargraph plot of singletons stats
            if len(self.bcftools_stats_sample_singletons) > 0:
                pconfig = {
                    "id": "bcftools-stats-singletons",
                    "title": "Bcftools Stats: Singletons",
                    "ylab": "# Singletons",
                    "cpswitch_counts_label": "Singletons",
                    "cpswitch_c_active": False,
                    "data_labels": list(self.bcftools_stats_sample_singletons),
                }
                self.add_section(
                    name="Number of Singletons",
                    anchor="bcftools-stats_singletones",
                    plot=bargraph.plot(
                        list(self.bcftools_stats_sample_singletons.values()), ["singletons", "rest"], pconfig
                    ),
                )

            # Make bargraph plot of sequencing depth stats
            if len(self.bcftools_stats_sample_depth) > 0:
                pconfig = {
                    "id": "bcftools-stats-depth",
                    "title": "Bcftools Stats: Sequencing depth",
                    "ylab": "Sequencing depth",
                    "cpswitch_counts_label": "Sequencing depth",
                    "cpswitch": False,
                    "data_labels": list(self.bcftools_stats_sample_depth),
                }
                self.add_section(
                    name="Sequencing depth",
                    anchor="bcftools-stats_sequencing_depth",
                    plot=bargraph.plot(list(self.bcftools_stats_sample_depth.values()), ["depth"], pconfig),
                )

        # Return the number of logs that were found
        return len(self.bcftools_stats)

    def bcftools_stats_genstats_headers(self):
        """Add key statistics to the General Stats table"""
        stats_headers = OrderedDict()
        stats_headers["number_of_records"] = {
            "title": "Vars",
            "description": "Variations total",
            "min": 0,
            "format": "{:,.0f}",
        }
        stats_headers["variations_hom"] = {
            "title": "Hom",
            "description": "Variations homozygous",
            "min": 0,
            "format": "{:,.0f}",
        }
        stats_headers["variations_het"] = {
            "title": "Het",
            "description": "Variations heterozygous",
            "min": 0,
            "format": "{:,.0f}",
        }
        stats_headers["number_of_SNPs"] = {
            "title": "SNP",
            "description": "Variation SNPs",
            "min": 0,
            "format": "{:,.0f}",
        }
        stats_headers["number_of_indels"] = {
            "title": "Indel",
            "description": "Variation Insertions/Deletions",
            "min": 0,
            "format": "{:,.0f}",
        }
        stats_headers["tstv"] = {
            "title": "Ts/Tv",
            "description": "Variant SNP transition / transversion ratio",
            "min": 0,
            "format": "{:,.2f}",
        }
        stats_headers["number_of_MNPs"] = {
            "title": "MNP",
            "description": "Variation multinucleotide polymorphisms",
            "min": 0,
            "format": "{:,.0f}",
            "hidden": True,
        }
        return stats_headers



#!/usr/bin/env python

""" MultiQC module to parse output from Picard """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc.modules.base_module import BaseMultiqcModule

# Import the Picard submodules
from . import AlignmentSummaryMetrics
from . import BaseDistributionByCycleMetrics
from . import CollectIlluminaBasecallingMetrics
from . import CollectIlluminaLaneMetrics
from . import CrosscheckFingerprints
from . import ExtractIlluminaBarcodes
from . import GcBiasMetrics
from . import HsMetrics
from . import InsertSizeMetrics
from . import MarkDuplicates
from . import MarkIlluminaAdapters
from . import OxoGMetrics
from . import QualityByCycleMetrics
from . import QualityScoreDistributionMetrics
from . import QualityYieldMetrics
from . import RnaSeqMetrics
from . import RrbsSummaryMetrics
from . import TargetedPcrMetrics
from . import ValidateSamFile
from . import VariantCallingMetrics
from . import WgsMetrics


# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """Picard is a collection of scripts. This MultiQC module
    supports some but not all. The code for each script is split
    into its own file and adds a section to the module output if
    logs are found."""

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name="Picard",
            anchor="picard",
            href="http://broadinstitute.github.io/picard/",
            info="is a set of Java command line tools for manipulating high-" "throughput sequencing data.",
            # No DOI to cite // doi=
        )

        # Set up class objects to hold parsed data
        self.general_stats_headers = OrderedDict()
        self.general_stats_data = dict()
        n = dict()

        # Call submodule functions
        n["AlignmentMetrics"] = AlignmentSummaryMetrics.parse_reports(self)
        if n["AlignmentMetrics"] > 0:
            log.info("Found {} AlignmentSummaryMetrics reports".format(n["AlignmentMetrics"]))

        n["BaseDistributionByCycleMetrics"] = BaseDistributionByCycleMetrics.parse_reports(self)
        if n["BaseDistributionByCycleMetrics"] > 0:
            log.info("Found {} BaseDistributionByCycleMetrics reports".format(n["BaseDistributionByCycleMetrics"]))

        n["CrosscheckFingerprints"] = CrosscheckFingerprints.parse_reports(self)
        if n["CrosscheckFingerprints"] > 0:
            log.info("Found {} CrosscheckFingerprints reports".format(n["CrosscheckFingerprints"]))

        n["GcBiasMetrics"] = GcBiasMetrics.parse_reports(self)
        if n["GcBiasMetrics"] > 0:
            log.info("Found {} GcBiasMetrics reports".format(n["GcBiasMetrics"]))

        n["HsMetrics"] = HsMetrics.parse_reports(self)
        if n["HsMetrics"] > 0:
            log.info("Found {} HsMetrics reports".format(n["HsMetrics"]))

        n["InsertSizeMetrics"] = InsertSizeMetrics.parse_reports(self)
        if n["InsertSizeMetrics"] > 0:
            log.info("Found {} InsertSizeMetrics reports".format(n["InsertSizeMetrics"]))

        n["MarkDuplicates"] = MarkDuplicates.parse_reports(self)
        if n["MarkDuplicates"] > 0:
            log.info("Found {} MarkDuplicates reports".format(n["MarkDuplicates"]))

        n["OxoGMetrics"] = OxoGMetrics.parse_reports(self)
        if n["OxoGMetrics"] > 0:
            log.info("Found {} OxoGMetrics reports".format(n["OxoGMetrics"]))

        n["QualityByCycleMetrics"] = QualityByCycleMetrics.parse_reports(self)
        if n["QualityByCycleMetrics"] > 0:
            log.info("Found {} QualityByCycleMetrics reports".format(n["QualityByCycleMetrics"]))

        n["QualityScoreDistributionMetrics"] = QualityScoreDistributionMetrics.parse_reports(self)
        if n["QualityScoreDistributionMetrics"] > 0:
            log.info("Found {} QualityScoreDistributionMetrics reports".format(n["QualityScoreDistributionMetrics"]))

        n["QualityYieldMetrics"] = QualityYieldMetrics.parse_reports(self)
        if n["QualityYieldMetrics"] > 0:
            log.info("Found {} QualityYieldMetrics reports".format(n["QualityYieldMetrics"]))

        n["RnaSeqMetrics"] = RnaSeqMetrics.parse_reports(self)
        if n["RnaSeqMetrics"] > 0:
            log.info("Found {} RnaSeqMetrics reports".format(n["RnaSeqMetrics"]))

        n["RrbsSummaryMetrics"] = RrbsSummaryMetrics.parse_reports(self)
        if n["RrbsSummaryMetrics"] > 0:
            log.info("Found {} RrbsSummaryMetrics reports".format(n["RrbsSummaryMetrics"]))

        n["TargetedPcrMetrics"] = TargetedPcrMetrics.parse_reports(self)
        if n["TargetedPcrMetrics"] > 0:
            log.info("Found {} TargetedPcrMetrics reports".format(n["TargetedPcrMetrics"]))

        n["VariantCallingMetrics"] = VariantCallingMetrics.parse_reports(self)
        if n["VariantCallingMetrics"] > 0:
            log.info("Found {} VariantCallingMetrics reports".format(n["VariantCallingMetrics"]))

        n["ValidateSamFile"] = ValidateSamFile.parse_reports(self)
        if n["ValidateSamFile"] > 0:
            log.info("Found {} ValidateSamFile reports".format(n["ValidateSamFile"]))

        n["WgsMetrics"] = WgsMetrics.parse_reports(self)
        if n["WgsMetrics"] > 0:
            log.info("Found {} WgsMetrics reports".format(n["WgsMetrics"]))

        n["CollectIlluminaBasecallingMetrics"] = CollectIlluminaBasecallingMetrics.parse_reports(self)
        if n["CollectIlluminaBasecallingMetrics"] > 0:
            log.info(
                "Found {} CollectIlluminaBasecallingMetrics reports".format(n["CollectIlluminaBasecallingMetrics"])
            )

        n["CollectIlluminaLaneMetrics"] = CollectIlluminaLaneMetrics.parse_reports(self)
        if n["CollectIlluminaLaneMetrics"] > 0:
            log.info("Found {} CollectIlluminaLaneMetrics reports".format(n["CollectIlluminaLaneMetrics"]))

        n["ExtractIlluminaBarcodes"] = ExtractIlluminaBarcodes.parse_reports(self)
        if n["ExtractIlluminaBarcodes"] > 0:
            log.info("Found {} ExtractIlluminaBarcodes reports".format(n["ExtractIlluminaBarcodes"]))

        n["MarkIlluminaAdapters"] = MarkIlluminaAdapters.parse_reports(self)
        if n["MarkIlluminaAdapters"] > 0:
            log.info("Found {} MarkIlluminaAdapters reports".format(n["MarkIlluminaAdapters"]))

        # Exit if we didn't find anything
        if sum(n.values()) == 0:
            raise UserWarning

        # Add to the General Stats table (has to be called once per MultiQC module)
        self.general_stats_addcols(self.general_stats_data, self.general_stats_headers)

    # Helper functions
    def multiply_hundred(self, val):
        try:
            val = float(val) * 100
        except ValueError:
            pass
        return val



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard TargetedPcrMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard TargetedPcrMetrics reports and parse their data"""

    # Set up vars
    self.picard_pcrmetrics_data = dict()
    self.picard_pcrmetrics_samplestats = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/pcr_metrics", filehandles=True):
        s_name = None
        for l in f["f"]:
            # New log starting
            if "TargetedPcrMetrics" in l and "INPUT" in l:
                s_name = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)

            if s_name is not None:
                if "TargetedPcrMetrics" in l and "## METRICS CLASS" in l:
                    keys = f["f"].readline().strip("\n").split("\t")
                    vals = f["f"].readline().strip("\n").split("\t")
                    if len(vals) == len(keys):
                        if s_name in self.picard_pcrmetrics_data:
                            log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
                        self.add_data_source(f, s_name, section="TargetedPcrMetrics")
                        self.picard_pcrmetrics_data[s_name] = dict()
                        for i, k in enumerate(keys):
                            try:
                                # Multiply percentages by 100
                                if k.startswith("PCT_"):
                                    vals[i] = float(vals[i]) * 100.0
                                self.picard_pcrmetrics_data[s_name][k] = float(vals[i])
                            except ValueError:
                                self.picard_pcrmetrics_data[s_name][k] = vals[i]

    # Filter to strip out ignored sample names
    self.picard_pcrmetrics_data = self.ignore_samples(self.picard_pcrmetrics_data)

    if len(self.picard_pcrmetrics_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_pcrmetrics_data, "multiqc_picard_pcrmetrics")

        # Add to general stats table
        self.general_stats_headers["PCT_AMPLIFIED_BASES"] = {
            "title": "% Amplified Bases",
            "description": "The fraction of aligned bases that mapped to or near an amplicon.",
            "min": 0,
            "max": 100,
            "suffix": "%",
            "scale": "BrBG",
        }
        self.general_stats_headers["MEDIAN_TARGET_COVERAGE"] = {
            "title": "Median Target Coverage",
            "description": "The median coverage of reads that mapped to target regions of an experiment.",
            "min": 0,
            "suffix": "X",
            "scale": "GnBu",
        }

        for s_name in self.picard_pcrmetrics_data:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(self.picard_pcrmetrics_data[s_name])

        # Bar plot of ignored bases
        keys = OrderedDict()
        keys["ON_AMPLICON_BASES"] = {"name": "On-amplicon bases"}
        keys["NEAR_AMPLICON_BASES"] = {"name": "Near-amplicon bases"}
        keys["OFF_AMPLICON_BASES"] = {"name": "Off-amplicon bases", "color": "#f28f43"}

        # Config for the plot
        pconfig = {
            "id": "picard_pcr_metrics_bases",
            "title": "Picard: PCR Amplicon Bases",
            "ylab": "# Bases",
            "cpswitch_counts_label": "# Bases",
            "hide_zero_cats": False,
        }

        self.add_section(
            name="PCR Amplicon Bases",
            anchor="picard-pcrmetrics-bases",
            description="Metrics about reads obtained from targeted PCR experiments.",
            helptext="""
            This plot shows the number of bases aligned on or near to amplified regions of the genome.

            * `ON_AMPLICON_BASES`: The number of `PF_BASES_ALIGNED` that mapped to an amplified region of the genome.
            * `NEAR_AMPLICON_BASES`: The number of `PF_BASES_ALIGNED` that mapped to within a fixed interval of an amplified region, but not on a baited region.
            * `OFF_AMPLICON_BASES`: The number of `PF_BASES_ALIGNED` that mapped neither on or near an amplicon.

            For more information see the [Picard documentation](https://broadinstitute.github.io/picard/picard-metric-definitions.html#TargetedPcrMetrics).""",
            plot=bargraph.plot(self.picard_pcrmetrics_data, keys, pconfig),
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_pcrmetrics_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC submodule to parse output from GATK varianteval """

import logging
from collections import OrderedDict
from multiqc.plots import bargraph, table

# Initialise the logger
log = logging.getLogger(__name__)


class VariantEvalMixin:
    def parse_gatk_varianteval(self):
        """Find GATK varianteval logs and parse their data"""

        self.gatk_varianteval = dict()
        for f in self.find_log_files("gatk/varianteval", filehandles=True):
            parsed_data = parse_single_report(f["f"])
            if len(parsed_data) > 1:
                if f["s_name"] in self.gatk_varianteval:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f["s_name"]))
                self.add_data_source(f, section="varianteval")
                self.gatk_varianteval[f["s_name"]] = parsed_data

        # Filter to strip out ignored sample names
        self.gatk_varianteval = self.ignore_samples(self.gatk_varianteval)

        n_reports_found = len(self.gatk_varianteval)
        if n_reports_found > 0:
            log.info("Found {} VariantEval reports".format(n_reports_found))

            # Write parsed report data to a file (restructure first)
            self.write_data_file(self.gatk_varianteval, "multiqc_gatk_varianteval")

            # Get consensus TiTv references
            titv_ref = None
            for s_name in self.gatk_varianteval:
                if titv_ref is None:
                    titv_ref = self.gatk_varianteval[s_name]["titv_reference"]
                elif titv_ref != self.gatk_varianteval[s_name]["titv_reference"]:
                    titv_ref = "Multiple"
                    break

            # General Stats Table
            varianteval_headers = dict()
            varianteval_headers["known_titv"] = {
                "title": "TiTV ratio (known)",
                "description": "TiTV ratio from variants found in '{}'".format(titv_ref),
                "min": 0,
                "scale": "Blues",
                "shared_key": "titv_ratio",
            }
            varianteval_headers["novel_titv"] = {
                "title": "TiTV ratio (novel)",
                "description": "TiTV ratio from variants NOT found in '{}'".format(titv_ref),
                "min": 0,
                "scale": "Blues",
                "shared_key": "titv_ratio",
            }
            varianteval_headers["called_titv"] = {
                "title": "TiTV ratio (called)",
                "description": "TiTV ratio from variants found in '{}'".format(titv_ref),
                "min": 0,
                "scale": "Blues",
                "shared_key": "titv_ratio",
            }
            varianteval_headers["filtered_titv"] = {
                "title": "TiTV ratio (filtered)",
                "description": "TiTV ratio from variants NOT found in '{}'".format(titv_ref),
                "min": 0,
                "scale": "Blues",
                "shared_key": "titv_ratio",
            }
            self.general_stats_addcols(self.gatk_varianteval, varianteval_headers, "GATK VariantEval")

            # Variant Counts plot
            self.add_section(
                name="Variant Counts", anchor="gatk-count-variants", plot=count_variants_barplot(self.gatk_varianteval)
            )

            # Compare Overlap Table
            self.add_section(
                name="Compare Overlap", anchor="gatk-compare-overlap", plot=comp_overlap_table(self.gatk_varianteval)
            )

        return n_reports_found


def parse_single_report(f):
    """Parse a gatk varianteval varianteval"""
    # Fixme: Separate GATKReport parsing and data subsetting. A GATKReport parser now available from the GATK MultiqcModel.

    data = dict()
    in_CompOverlap = False
    in_CountVariants = False
    in_TiTv = False
    for l in f:
        # Detect section headers
        if "#:GATKTable:CompOverlap" in l:
            in_CompOverlap = True
        elif "#:GATKTable:CountVariants" in l:
            in_CountVariants = True
        elif "#:GATKTable:TiTvVariantEvaluator" in l:
            in_TiTv = True
        else:
            # Parse contents using nested loops
            if in_CompOverlap:
                headers = l.split()
                while in_CompOverlap:
                    l = f.readline().strip("\n")
                    if len(l) < len(headers):
                        in_CompOverlap = False
                        break
                    d = dict()
                    for i, s in enumerate(l.split()):
                        d[headers[i]] = s
                    if d.get("Novelty") == "all" or d.get("Filter") == "raw":
                        if "CompFeatureInput" in d:
                            data["reference"] = d["CompFeatureInput"]
                        if "CompRod" in d:
                            data["reference"] = d["CompRod"]
                        if "compRate" in d:
                            data["comp_rate"] = float(d["compRate"])
                        if "concordantRate" in d:
                            data["concordant_rate"] = float(d["concordantRate"])
                        if "nEvalVariants" in d:
                            data["eval_variants"] = int(d["nEvalVariants"])
                        if "novelSites" in d:
                            data["novel_sites"] = int(d["novelSites"])
                    elif d.get("Novelty") == "known":
                        if "nEvalVariants" in d:
                            data["known_sites"] = int(d["nEvalVariants"])

            elif in_CountVariants:
                headers = l.split()
                while in_CountVariants:
                    l = f.readline().strip("\n")
                    if len(l) < len(headers):
                        in_CountVariants = False
                        break
                    d = dict()
                    for i, s in enumerate(l.split()):
                        d[headers[i]] = s
                    if d.get("Novelty") == "all" or d.get("Filter") == "raw":
                        if "nSNPs" in d:
                            data["snps"] = int(d["nSNPs"])
                        if "nMNPs" in d:
                            data["mnps"] = int(d["nMNPs"])
                        if "nInsertions" in d:
                            data["insertions"] = int(d["nInsertions"])
                        if "nDeletions" in d:
                            data["deletions"] = int(d["nDeletions"])
                        if "nComplex" in d:
                            data["complex"] = int(d["nComplex"])
                        if "nSymbolic" in d:
                            data["symbolic"] = int(d["nSymbolic"])
                        if "nMixed" in d:
                            data["mixed"] = int(d["nMixed"])
                        if "nNoCalls" in d:
                            data["nocalls"] = int(d["nNoCalls"])
            elif in_TiTv:
                headers = l.split()
                data["titv_reference"] = "unknown"
                while in_TiTv:
                    l = f.readline().strip("\n")
                    if len(l) < len(headers):
                        in_TiTv = False
                        break
                    d = dict()
                    for i, s in enumerate(l.split()):
                        d[headers[i]] = s
                    if d.get("Novelty") == "known":
                        if "CompFeatureInput" in d:
                            data["titv_reference"] = d["CompFeatureInput"]
                        if "CompRod" in d:
                            data["titv_reference"] = d["CompRod"]
                        if "tiTvRatio" in d:
                            data["known_titv"] = float(d["tiTvRatio"])
                    elif d.get("Novelty") == "novel":
                        if "tiTvRatio" in d:
                            data["novel_titv"] = float(d["tiTvRatio"])
                    elif d.get("Filter") == "called":
                        if "CompFeatureInput" in d:
                            data["titv_reference"] = d["CompFeatureInput"]
                        if "CompRod" in d:
                            data["titv_reference"] = d["CompRod"]
                        if "tiTvRatio" in d:
                            data["called_titv"] = float(d["tiTvRatio"])
                    elif d.get("Filter") == "filtered":
                        if "CompFeatureInput" in d:
                            data["titv_reference"] = d["CompFeatureInput"]
                        if "CompRod" in d:
                            data["titv_reference"] = d["CompRod"]
                        if "tiTvRatio" in d:
                            data["filtered_titv"] = float(d["tiTvRatio"])

    return data


def count_variants_barplot(data):
    """Return HTML for the Variant Counts barplot"""
    keys = OrderedDict()
    keys["snps"] = {"name": "SNPs"}
    keys["mnps"] = {"name": "MNPs"}
    keys["insertions"] = {"name": "Insertions"}
    keys["deletions"] = {"name": "Deletions"}
    keys["complex"] = {"name": "Complex"}
    keys["symbolic"] = {"name": "Symbolic"}
    keys["mixed"] = {"name": "Mixed"}
    keys["nocalls"] = {"name": "No-calls"}

    plot_conf = {
        "id": "gatk_varianteval_variant_plot",
        "title": "GATK VariantEval: Variant Counts",
        "ylab": "# Variants",
        "cpswitch_counts_label": "Number of Variants",
    }
    return bargraph.plot(data, keys, plot_conf)


def comp_overlap_table(data):
    """Build a table from the comp overlaps output."""
    headers = OrderedDict()
    headers["comp_rate"] = {
        "title": "Compare rate",
        "description": "Ratio of known variants found in the reference set.",
        "namespace": "GATK",
        "min": 0,
        "max": 100,
        "suffix": "%",
        "format": "{:,.2f}",
        "scale": "Blues",
    }
    headers["concordant_rate"] = {
        "title": "Concordant rate",
        "description": "Ratio of variants matching alleles in the reference set.",
        "namespace": "GATK",
        "min": 0,
        "max": 100,
        "suffix": "%",
        "format": "{:,.2f}",
        "scale": "Blues",
    }
    headers["eval_variants"] = {
        "title": "M Evaluated variants",
        "description": "Number of called variants (millions)",
        "namespace": "GATK",
        "min": 0,
        "modify": lambda x: float(x) / 1000000.0,
    }
    headers["known_sites"] = {
        "title": "M Known sites",
        "description": "Number of known variants (millions)",
        "namespace": "GATK",
        "min": 0,
        "modify": lambda x: float(x) / 1000000.0,
    }
    headers["novel_sites"] = {
        "title": "M Novel sites",
        "description": "Number of novel variants (millions)",
        "namespace": "GATK",
        "min": 0,
        "modify": lambda x: float(x) / 1000000.0,
    }
    table_html = table.plot(data, headers, {"id": "gatk_compare_overlap", "table_title": "GATK - Compare Overlap"})
    return table_html



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard CollectIlluminaLaneMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import table

# Initialise the logger
log = logging.getLogger(__name__)


def lane_metrics_table(data):
    headers = OrderedDict()
    headers["CLUSTER_DENSITY"] = {
        "title": "Cluster Density",
        "description": "The number of clusters per unit area on the this lane (cluster / mm^2`)",
        "scale": "Greens",
    }
    headers["TYPE_NAME"] = {
        "title": "Read Number",
        "description": "Defines an Illumina template read number (first or second)",
        "modify": lambda x: x.lower(),
    }
    headers["PREPHASING_APPLIED"] = {
        "title": "Prephasing Applied",
        "description": "Median pre-phasing value across all tiles in a lane, applied to the first and second template reads",
        "scale": "BuPu",
        "max": 1,
    }
    headers["PHASING_APPLIED"] = {
        "title": "Phasing Applied",
        "description": "Median phasing value across all tiles in a lane, applied to the first and second template reads",
        "scale": "BuPu",
        "max": 1,
    }

    table_config = {
        "id": "picard-illumina-lane-metrics-table",
        "namespace": "Picard",
        "table_title": "Picard Illumina Lane Metrics",
    }
    tdata = {}
    for run_name, run in data.items():
        for lane_number, lane in run.items():
            tdata[f"{run_name} - L{lane_number}"] = lane
    return table.plot(tdata, headers, table_config)


def parse_reports(self):
    """Find Picard CollectIlluminaLaneMetrics reports and parse their data"""

    # Set up vars
    self.picard_lane_metrics = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/collectilluminalanemetrics", filehandles=True):
        self.add_data_source(f, section="CollectIlluminaLaneMetrics")

        run_name = None
        parsed_data = []
        keys = []

        for line in f["f"]:
            if "CollectIlluminaLaneMetrics" in line and "RUN_DIRECTORY" in line:
                run_dir_search = re.search(r"RUN_DIRECTORY(?:=|\s+)(\[?[^\s]+\]?)", line, flags=re.IGNORECASE)
                run_name = os.path.basename(run_dir_search[0])
                if not run_name in self.picard_lane_metrics:
                    self.picard_lane_metrics[run_name] = {}

            if run_name is not None:
                if ("IlluminaLaneMetrics" in line or "IlluminaPhasingMetrics" in line) and "## METRICS CLASS" in line:
                    keys = f["f"].readline().strip("\n").split("\t")
                elif keys:
                    vals = line.strip("\n").split("\t")
                    if len(vals) == len(keys):
                        parsed_data.append(dict(zip(keys, vals)))

                for d in parsed_data:
                    lane = d["LANE"]
                    if not lane in self.picard_lane_metrics[run_name]:
                        self.picard_lane_metrics[run_name][lane] = {}
                    self.picard_lane_metrics[run_name][lane].update(d)

    # Filter to strip out ignored sample names
    self.picard_lane_metrics = self.ignore_samples(self.picard_lane_metrics)

    if len(self.picard_lane_metrics) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_lane_metrics, "multiqc_picard_IlluminaLaneMetrics")

        self.add_section(
            name="Lane Metrics",
            anchor="picard-illuminalanemetrics",
            description="Quality control metrics on cluster density for each lane of an Illumina flowcell. For more information, see the [Picard Documentation](https://broadinstitute.github.io/picard/picard-metric-definitions.html#IlluminaLaneMetrics).",
            plot=lane_metrics_table(self.picard_lane_metrics),
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_lane_metrics)



#!/usr/bin/env python

""" MultiQC module to parse output from NanoStat """

from __future__ import print_function
from collections import OrderedDict
import logging
import jinja2

from multiqc import config
from multiqc.utils import mqc_colour
from multiqc.plots import bargraph, table
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """NanoStat module"""

    _KEYS_NUM = [
        "Active channels",
        "Number of reads",
        "Total bases",
        "Total bases aligned",
        "Read length N50",
        "Mean read length",
        "Median read length",
        "Median read quality",
        "Mean read quality",
        "Average percent identity",
        "Median percent identity",
    ]

    _KEYS_READ_Q = [
        "&gt;Q5",
        "&gt;Q7",
        "&gt;Q10",
        "&gt;Q12",
        "&gt;Q15",
    ]
    _stat_types = ("aligned", "seq summary", "fastq", "fasta", "unrecognized")

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name="NanoStat",
            anchor="nanostat",
            href="https://github.com/wdecoster/nanostat/",
            info="various statistics from a long read sequencing dataset in fastq, bam or sequencing summary format.",
            doi="10.1093/bioinformatics/bty149",
        )

        # Find and load any NanoStat reports
        self.nanostat_data = dict()
        self.has_aligned = False
        self.has_seq_summary = False
        self.has_fastq = False
        self.has_fasta = False
        for f in self.find_log_files("nanostat", filehandles=True):
            self.parse_nanostat_log(f)

        # Filter to strip out ignored sample names
        self.nanostat_data = self.ignore_samples(self.nanostat_data)

        if len(self.nanostat_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.nanostat_data)))

        # Write parsed report data to a file
        self.write_data_file(self.nanostat_data, "multiqc_nanostat")

        # Stats Tables
        if self.has_aligned:
            self.nanostat_stats_table("aligned")
        if self.has_seq_summary:
            self.nanostat_stats_table("seq summary")
        if self.has_fastq:
            self.nanostat_stats_table("fastq")
        if self.has_fasta:
            self.nanostat_stats_table("fasta")

        # Quality distribution Plot
        if self.has_aligned or self.has_seq_summary or self.has_fastq:
            self.reads_by_quality_plot()

    def parse_nanostat_log(self, f):
        """Parse output from NanoStat

        Note: Tool can be run in two different modes, giving two variants to the output.
        To avoid overwriting keys from different modes, keys are given a suffix.
        """

        nano_stats = {}
        for line in f["f"]:

            line = jinja2.escape(line)
            parts = line.strip().split(":")
            if len(parts) == 0:
                continue

            key = parts[0]

            if key in self._KEYS_NUM:
                val = float(parts[1].replace(",", ""))
                nano_stats[key] = val
            elif key in self._KEYS_READ_Q:
                # Number of reads above Q score cutoff
                val = int(parts[1].strip().split()[0])
                nano_stats[key] = val

        if "Total bases aligned" in nano_stats:
            stat_type = "aligned"
            self.has_aligned = True
        elif "Active channels" in nano_stats:
            stat_type = "seq summary"
            self.has_seq_summary = True
        elif "Mean read quality" in nano_stats:
            stat_type = "fastq"
            self.has_fastq = True
        elif "Mean read length" in nano_stats:
            stat_type = "fasta"
            self.has_fasta = True
        else:
            log.debug(f"Did not recognise NanoStat file '{f['fn']}' - skipping")
            return

        out_d = {f"{k}_{stat_type}": v for k, v in nano_stats.items()}

        # Warn if we find overlapping data for the same sample
        if f["s_name"] in self.nanostat_data:
            # Only if the same has some keys in common
            if not set(self.nanostat_data[f["s_name"]].keys()).isdisjoint(out_d.keys()):
                log.debug("Duplicate sample data found! Overwriting: {}".format(f["s_name"]))

        self.nanostat_data.setdefault(f["s_name"], {}).update(out_d)

        self.add_data_source(f)

    def nanostat_stats_table(self, stat_type):
        """Take the parsed stats from the Kallisto report and add it to the
        basic stats table at the top of the report"""

        headers_base = OrderedDict()
        headers_base["Active channels"] = {
            "title": "Active channels",
            "description": "Active channels",
            "scale": "Greens",
            "format": "{:,.0f}",
        }
        headers_base["Median read length"] = {
            "title": f"Median length",
            "description": f"Median read length (bp)",
            "suffix": " bp",
            "format": "{:,.0f}",
            "shared_key": "nucleotides",
            "scale": "BuPu",
        }
        headers_base["Mean read length"] = {
            "title": f"Mean length",
            "description": f"Mean read length (bp)",
            "suffix": " bp",
            "scale": "Purples",
            "format": "{:,.0f}",
            "shared_key": "nucleotides",
            "hidden": True,
        }
        headers_base["Read length N50"] = {
            "title": "Read N50",
            "description": "Read length N50",
            "format": "{:,.0f}",
            "suffix": " bp",
            "scale": "RdPu",
        }
        headers_base["Median read quality"] = {
            "title": "Median Qual",
            "description": "Median read quality (Phred scale)",
            "shared_key": "phred_score",
            "scale": "RdYlGn",
        }
        headers_base["Mean read quality"] = {
            "title": "Mean Qual",
            "description": "Mean read quality (Phred scale)",
            "scale": "PiYG",
            "shared_key": "phred_score",
            "hidden": True,
        }
        headers_base["Median percent identity"] = {
            "title": "Median Identity",
            "description": "Median percent identity",
            "min": 0,
            "max": 100,
            "suffix": "%",
            "scale": "RdYlBu",
            "shared_key": "percent_identity",
        }
        headers_base["Average percent identity"] = {
            "title": "Mean Identity",
            "description": "Average percent identity",
            "max": 100,
            "suffix": "%",
            "scale": "Spectral",
            "shared_key": "percent_identity",
            "hidden": True,
        }
        headers_base["Number of reads"] = {
            "title": f"# Reads ({config.long_read_count_prefix})",
            "description": f"Number of reads ({config.long_read_count_desc})",
            "modify": lambda x: x * config.long_read_count_multiplier,
            "shared_key": "long_read_count",
            "scale": "YlGn",
        }
        headers_base["Total bases"] = {
            "title": f"Total Bases ({config.base_count_prefix})",
            "description": f"Total bases ({config.base_count_desc})",
            "modify": lambda x: x * config.base_count_multiplier,
            "shared_key": "base_count",
            "scale": "BrBG",
        }
        headers_base["Total bases aligned"] = {
            "title": f"Aligned Bases ({config.base_count_prefix})",
            "description": f"Total bases aligned ({config.base_count_desc})",
            "modify": lambda x: x * config.base_count_multiplier,
            "shared_key": "base_count",
            "scale": "PuOr",
        }

        # Add the stat_type suffix
        headers = OrderedDict()
        for k in headers_base:
            key = f"{k}_{stat_type}"
            headers[key] = headers_base.get(k, dict()).copy()

        # Table config
        table_config = {
            "namespace": "NanoStat",
            "id": "nanostat_{}_stats_table".format(stat_type.replace(" ", "_")),
            "table_title": f"NanoStat {stat_type}",
        }

        # Add the report section
        description = ""
        if stat_type == "fasta":
            description = "NanoStat statistics from FASTA files."
        if stat_type == "fastq":
            description = "NanoStat statistics from FastQ files."
        if stat_type == "aligned":
            description = "NanoStat statistics from BAM files."
        if stat_type == "seq summary":
            description = "NanoStat statistics from albacore or guppy summary files."

        self.add_section(
            name="{} stats".format(stat_type.replace("_", " ").capitalize()),
            anchor="nanostat_{}_stats".format(stat_type.replace(" ", "_")),
            description=description,
            plot=table.plot(self.nanostat_data, headers, table_config),
        )

    def reads_by_quality_plot(self):
        """Make the HighCharts HTML to plot the reads by quality"""

        def _get_total_reads(data_dict):
            stat_type = self._stat_types[0]
            for stat_type in self._stat_types:
                total_key = f"Number of reads_{stat_type}"
                if total_key in data_dict:
                    return data_dict[total_key], stat_type
            return None, None

        bar_data = {}
        stat_type = "unrecognized"
        # Order of keys, from >Q5 to >Q15
        _range_names = {
            "&gt;Q5": "&lt;Q5",
            "&gt;Q7": "Q5-7",
            "&gt;Q10": "Q7-10",
            "&gt;Q12": "Q10-12",
            "&gt;Q15": "Q12-15",
            "rest": "&gt;Q15",
        }
        for s_name, data_dict in self.nanostat_data.items():
            reads_total, stat_type = _get_total_reads(data_dict)
            if stat_type == "fasta":
                log.debug(f"Sample '{s_name}' has no quality metrics - excluded from quality plot")
                continue
            if s_name in bar_data and stat_type == "aligned":
                log.debug(f"Sample '{s_name}' duplicated in the quality plot - ignoring aligned data")
                continue
            elif s_name in bar_data and stat_type == "seq summary":
                log.debug(f"Sample '{s_name}' duplicated in the quality plot - overwriting with seq summary data")
            bar_data[s_name] = {}

            prev_reads = reads_total
            for k, range_name in _range_names.items():
                if k != "rest":
                    data_key = f"{k}_{stat_type}"
                    reads_gt = data_dict[data_key]

                    bar_data[s_name][range_name] = prev_reads - reads_gt

                    if bar_data[s_name][range_name] < 0:
                        log.error(f"Error on {s_name} {range_name} {data_key} . Negative number of reads")
                    prev_reads = reads_gt
                else:
                    data_key = f"&gt;Q15_{stat_type}"
                    bar_data[s_name][range_name] = data_dict[data_key]

        cats = OrderedDict()
        keys = reversed(list(_range_names.values()))
        colours = mqc_colour.mqc_colour_scale("RdYlGn-rev", 0, len(_range_names))
        for idx, k in enumerate(keys):
            cats[k] = {"name": "Reads " + k, "color": colours.get_colour(idx, lighten=1)}

        # Config for the plot
        config = {
            "id": "nanostat_quality_dist",
            "title": "NanoStat: Reads by quality",
            "ylab": "# Reads",
            "cpswitch_counts_label": "Number of Reads",
        }

        # Add the report section
        self.add_section(
            name="Reads by quality",
            anchor=f"nanostat_read_qualities",
            description="Read counts categorised by read quality (phred score).",
            helptext="""
                Sequencing machines assign each generated read a quality score using the
                [Phred scale](https://en.wikipedia.org/wiki/Phred_quality_score).
                The phred score represents the liklelyhood that a given read contains errors.
                So, high quality reads have a high score.

                Data may come from NanoPlot reports generated with sequencing summary files or alignment stats.
                If a sample has data from both, the sequencing summary is preferred.
            """,
            plot=bargraph.plot(bar_data, cats, config),
        )



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard BaseDistributionByCycleMetrics """

import logging
import os
import re

from multiqc.plots import linegraph
from .util import read_sample_name

# Initialise the logger
log = logging.getLogger(__name__)


def read_base_distrib_data(line_iter):
    """
    Consumes lines from the provided line_iter and parses those lines
    for base distribution data.  Data should be tab separated and
    immediately preceded by a line of headers:

    READ_END  CYCLE  PCT_A  PCT_C  PCT_G  PCT_T  PCT_N

    Returns either None or a dict mapping cycles to tuples
      (read_end pct_a pct_c pct_g pct_t pct_n)
    where all values are numbers.

    A None indicates that no lines matching the expected format
    were found.
    """
    try:
        line = next(line_iter)
        while "BaseDistributionByCycle" not in line and "## METRICS CLASS" not in line:
            line = next(line_iter)
        line = next(line_iter)
        headers = line.strip().split("\t")
        assert headers == ["READ_END", "CYCLE", "PCT_A", "PCT_C", "PCT_G", "PCT_T", "PCT_N"]

        # read base distribution by cycle
        data = {}

        row = next(line_iter).strip()
        max_cycle_r1 = None
        while row:
            row_data = list(map(float, row.strip().split("\t")))
            read_end, cycle, pct_a, pct_c, pct_g, pct_t, pct_n = row_data
            cycle = int(cycle)
            if read_end == 1.0:
                if max_cycle_r1 is None or cycle > max_cycle_r1:
                    max_cycle_r1 = cycle
            elif max_cycle_r1 is not None:
                cycle = cycle - max_cycle_r1
            data_by_cycle = data.setdefault(read_end, dict())
            data_by_cycle[cycle] = (pct_a, pct_c, pct_g, pct_t, pct_n)
            row = next(line_iter).strip()
        return data
    except (StopIteration, AssertionError):
        return None


def parse_reports(self):
    """Find Picard BaseDistributionByCycleMetrics reports and parse their data"""

    # Set up vars
    self.picard_baseDistributionByCycle_data = dict()
    self.picard_baseDistributionByCycle_samplestats = dict()

    # Go through logs and find Metrics
    base_dist_files = self.find_log_files("picard/basedistributionbycycle", filehandles=True)

    for f in base_dist_files:
        try:
            lines = iter(f["f"])

            # read through the header of the file to obtain the
            # sample name
            clean_fn = lambda n: self.clean_s_name(n, f)
            s_name = read_sample_name(lines, clean_fn, "BaseDistributionByCycle")
            assert s_name is not None

            # pull out the data
            data = read_base_distrib_data(lines)
            assert data is not None

            # data should be a hierarchical dict
            # data[read_end][cycle]
            assert not (set(data) - set([1, 2]))

            # set up the set of s_names
            if 2 in set(data):
                s_names = {1: "%s_R1" % s_name, 2: "%s_R2" % s_name}
            else:
                s_names = {1: s_name}

            previously_used = set(s_names.values()) & set(self.picard_baseDistributionByCycle_data)

            if previously_used:
                for duped_name in previously_used:
                    log.debug("Duplicate sample name found in {}! " "Overwriting: {}".format(f["fn"], duped_name))
            for name in s_names.values():
                self.add_data_source(f, name, section="BaseDistributionByCycle")

            for read_end in s_names:
                data_by_cycle = data[read_end]
                s_name = s_names[read_end]
                self.picard_baseDistributionByCycle_data[s_name] = data_by_cycle
                samplestats = {
                    "sum_pct_a": 0,
                    "sum_pct_c": 0,
                    "sum_pct_g": 0,
                    "sum_pct_t": 0,
                    "sum_pct_n": 0,
                    "cycle_count": 0,
                }
                self.picard_baseDistributionByCycle_samplestats[s_name] = samplestats
                for c, row in data_by_cycle.items():
                    pct_a, pct_c, pct_g, pct_t, pct_n = row
                    samplestats["sum_pct_a"] += pct_a
                    samplestats["sum_pct_c"] += pct_c
                    samplestats["sum_pct_g"] += pct_g
                    samplestats["sum_pct_t"] += pct_t
                    samplestats["sum_pct_n"] += pct_n
                samplestats["cycle_count"] += len(data_by_cycle.keys())
        except AssertionError:
            pass

    # Calculate summed mean values for all read orientations
    for s_name, v in self.picard_baseDistributionByCycle_samplestats.items():
        v["mean_pct_a"] = v["sum_pct_a"] / v["cycle_count"]
        v["mean_pct_c"] = v["sum_pct_c"] / v["cycle_count"]
        v["mean_pct_g"] = v["sum_pct_g"] / v["cycle_count"]
        v["mean_pct_t"] = v["sum_pct_t"] / v["cycle_count"]

    # Filter to strip out ignored sample names
    self.picard_baseDistributionByCycle_data = self.ignore_samples(self.picard_baseDistributionByCycle_data)

    if len(self.picard_baseDistributionByCycle_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_baseDistributionByCycle_samplestats, "multiqc_picard_baseContent")

        # Plot the data and add section
        pconfig = {
            "id": "picard_base_distribution_by_cycle",
            "title": "Picard: Base Distribution",
            "ylab": "%",
            "xlab": "Cycle #",
            "xDecimals": False,
            "tt_label": "<b>cycle {point.x}</b>: {point.y:.2f} %",
            "ymax": 100,
            "ymin": 0,
            "data_labels": [
                {"name": "% Adenine", "ylab": "% Adenine"},
                {"name": "% Cytosine", "ylab": "% Cytosine"},
                {"name": "% Guanine", "ylab": "% Guanine"},
                {"name": "% Thymine", "ylab": "% Thymine"},
                {"name": "% Undetermined", "ylab": "% Undetermined"},
            ],
        }

        # build list of linegraphs
        linegraph_data = [{}, {}, {}, {}, {}]
        for s_name, cycles in self.picard_baseDistributionByCycle_data.items():
            reformat_items = lambda n: {cycle: tup[n] for cycle, tup in cycles.items()}
            for lg, index in zip(linegraph_data, range(5)):
                lg[s_name] = reformat_items(index)

        self.add_section(
            name="Base Distribution",
            anchor="picard-base-distribution-by-cycle",
            description="Plot shows the distribution of bases by cycle.",
            plot=linegraph.plot(linegraph_data, pconfig),
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_baseDistributionByCycle_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard InsertSizeMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc import config
from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard InsertSizeMetrics reports and parse their data"""

    # Set up vars
    self.picard_insertSize_data = dict()
    self.picard_insertSize_histogram = dict()
    self.picard_insertSize_samplestats = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/insertsize", filehandles=True):
        s_name = None
        in_hist = False
        for l in f["f"]:

            # Catch the histogram values
            if s_name is not None and in_hist is True:
                try:
                    sections = l.split("\t")
                    ins = int(sections[0])
                    tot_count = sum([int(x) for x in sections[1:]])
                    self.picard_insertSize_histogram[s_name][ins] = tot_count
                    self.picard_insertSize_samplestats[s_name]["total_count"] += tot_count
                except ValueError:
                    # Reset in case we have more in this log file
                    s_name = None
                    in_hist = False

            # New log starting
            if "InsertSizeMetrics" in l and "INPUT" in l:
                s_name = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)

            if s_name is not None:
                if "InsertSizeMetrics" in l and "## METRICS CLASS" in l:
                    if s_name in self.picard_insertSize_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
                    self.add_data_source(f, s_name, section="InsertSizeMetrics")
                    keys = f["f"].readline().strip("\n").split("\t")
                    vals = f["f"].readline().strip("\n").split("\t")
                    self.picard_insertSize_samplestats[s_name] = {"total_count": 0, "meansum": 0, "total_pairs": 0}
                    orientation_idx = keys.index("PAIR_ORIENTATION")
                    while len(vals) == len(keys):
                        pair_orientation = vals[orientation_idx]
                        rowkey = "{}_{}".format(s_name, pair_orientation)
                        self.picard_insertSize_data[rowkey] = OrderedDict()
                        self.picard_insertSize_data[rowkey]["SAMPLE_NAME"] = s_name
                        for i, k in enumerate(keys):
                            try:
                                self.picard_insertSize_data[rowkey][k] = float(vals[i])
                            except ValueError:
                                try:
                                    self.picard_insertSize_data[rowkey][k] = float(vals[i].replace(",", "."))
                                    log.debug(
                                        "Switching commas for points in '{}': {} - {}".format(
                                            f["fn"], vals[i], vals[i].replace(",", ".")
                                        )
                                    )
                                except ValueError:
                                    self.picard_insertSize_data[rowkey][k] = vals[i]
                            except IndexError:
                                pass  # missing data
                        # Add to mean sums
                        rp = self.picard_insertSize_data[rowkey]["READ_PAIRS"]
                        mis = self.picard_insertSize_data[rowkey]["MEAN_INSERT_SIZE"]
                        self.picard_insertSize_samplestats[s_name]["meansum"] += rp * mis
                        self.picard_insertSize_samplestats[s_name]["total_pairs"] += rp

                        vals = f["f"].readline().strip("\n").split("\t")

                    # Skip lines on to histogram
                    l = f["f"].readline().strip("\n")
                    l = f["f"].readline().strip("\n")

                    self.picard_insertSize_histogram[s_name] = OrderedDict()
                    in_hist = True

        for key in list(self.picard_insertSize_data.keys()):
            if len(self.picard_insertSize_data[key]) == 0:
                self.picard_insertSize_data.pop(key, None)
        for s_name in list(self.picard_insertSize_histogram.keys()):
            if len(self.picard_insertSize_histogram[s_name]) == 0:
                self.picard_insertSize_histogram.pop(s_name, None)
                log.debug("Ignoring '{}' histogram as no data parsed".format(s_name))

    # Calculate summed mean values for all read orientations
    for s_name, v in self.picard_insertSize_samplestats.items():
        self.picard_insertSize_samplestats[s_name]["summed_mean"] = v["meansum"] / v["total_pairs"]

    # Calculate summed median values for all read orientations
    for s_name in self.picard_insertSize_histogram:
        j = 0
        for idx, c in self.picard_insertSize_histogram[s_name].items():
            j += c
            if j > (self.picard_insertSize_samplestats[s_name]["total_count"] / 2):
                self.picard_insertSize_samplestats[s_name]["summed_median"] = idx
                break

    # Filter to strip out ignored sample names
    self.picard_insertSize_data = self.ignore_samples(self.picard_insertSize_data)

    if len(self.picard_insertSize_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_insertSize_data, "multiqc_picard_insertSize")

        # Do we have median insert sizes?
        missing_medians = False
        for v in self.picard_insertSize_samplestats.values():
            if "summed_median" not in v:
                missing_medians = True

        # Add to general stats table
        self.general_stats_headers["summed_median"] = {
            "title": "Insert Size",
            "description": "Median Insert Size, all read orientations (bp)",
            "min": 0,
            "suffix": " bp",
            "format": "{:,.0f}",
            "scale": "GnBu",
        }
        self.general_stats_headers["summed_mean"] = {
            "title": "Mean Insert Size",
            "description": "Mean Insert Size, all read orientations (bp)",
            "min": 0,
            "suffix": " bp",
            "format": "{:,.0f}",
            "scale": "GnBu",
            "hidden": False if missing_medians else True,
        }
        for s_name in self.picard_insertSize_samplestats:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(self.picard_insertSize_samplestats[s_name])

        # Section with histogram plot
        if len(self.picard_insertSize_histogram) > 0:
            # Make a normalised percentage version of the data
            data_percent = {}
            for s_name, data in self.picard_insertSize_histogram.items():
                data_percent[s_name] = OrderedDict()
                total = float(sum(data.values()))
                for k, v in data.items():
                    data_percent[s_name][k] = (v / total) * 100

            # Allow customisation of how smooth the the plot is
            try:
                insertsize_smooth_points = int(config.picard_config["insertsize_smooth_points"])
                log.debug("Custom Picard insert size smoothing: {}".format(insertsize_smooth_points))
            except (AttributeError, KeyError, ValueError):
                insertsize_smooth_points = 500

            # Plot the data and add section
            pconfig = {
                "smooth_points": insertsize_smooth_points,
                "smooth_points_sumcounts": [True, False],
                "id": "picard_insert_size",
                "title": "Picard: Insert Size",
                "ylab": "Count",
                "xlab": "Insert Size (bp)",
                "xDecimals": False,
                "tt_label": "<b>{point.x} bp</b>: {point.y:.0f}",
                "ymin": 0,
                "data_labels": [
                    {"name": "Counts", "ylab": "Coverage"},
                    {"name": "Percentages", "ylab": "Percentage of Counts"},
                ],
            }
            try:
                pconfig["xmax"] = config.picard_config["insertsize_xmax"]
            except (AttributeError, KeyError):
                pass

            self.add_section(
                name="Insert Size",
                anchor="picard-insertsize",
                description="Plot shows the number of reads at a given insert size. Reads with different orientations are summed.",
                plot=linegraph.plot([self.picard_insertSize_histogram, data_percent], pconfig),
            )

    # Return the number of detected samples to the parent module
    return len(self.picard_insertSize_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard AlignmentSummaryMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard AlignmentSummaryMetrics reports and parse their data"""

    # Set up vars
    self.picard_alignment_metrics = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/alignment_metrics", filehandles=True):
        parsed_data = dict()
        s_name = None
        keys = None
        for l in f["f"]:
            # New log starting
            if "AlignmentSummaryMetrics" in l and "INPUT" in l:
                s_name = None
                keys = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)
                    parsed_data[s_name] = dict()

            if s_name is not None:
                if "AlignmentSummaryMetrics" in l and "## METRICS CLASS" in l:
                    keys = f["f"].readline().strip("\n").split("\t")
                elif keys:
                    vals = l.strip("\n").split("\t")
                    if len(vals) == len(keys):
                        # Ignore the FIRST_OF_PAIR / SECOND_OF_PAIR data to simplify things
                        if vals[0] == "PAIR" or vals[0] == "UNPAIRED":
                            for i, k in enumerate(keys):
                                try:
                                    parsed_data[s_name][k] = float(vals[i])
                                except ValueError:
                                    parsed_data[s_name][k] = vals[i]
                    else:
                        s_name = None
                        keys = None

        # Remove empty dictionaries
        for s_name in list(parsed_data.keys()):
            if len(parsed_data[s_name]) == 0:
                parsed_data.pop(s_name, None)

        # Manipulate sample names if multiple baits found
        for s_name in parsed_data.keys():
            if s_name in self.picard_alignment_metrics:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
            self.add_data_source(f, s_name, section="AlignmentSummaryMetrics")
            self.picard_alignment_metrics[s_name] = parsed_data[s_name]

    # Filter to strip out ignored sample names
    self.picard_alignment_metrics = self.ignore_samples(self.picard_alignment_metrics)

    if len(self.picard_alignment_metrics) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_alignment_metrics, "multiqc_picard_AlignmentSummaryMetrics")

        # Add to general stats table
        self.general_stats_headers["PCT_PF_READS_ALIGNED"] = {
            "title": "% Aligned",
            "description": "Percent of aligned reads",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "format": "{:,.0f}",
            "scale": "RdYlGn",
            "modify": lambda x: self.multiply_hundred(x),
        }
        for s_name in self.picard_alignment_metrics:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(self.picard_alignment_metrics[s_name])

        # Make the bar plot of alignment read count + # aligned bases
        pdata = dict()
        for s_name in self.picard_alignment_metrics.keys():
            pdata[s_name] = dict()
            # Picard reports both reads for PE data. Divide it by two as most people will expect # clusters
            if self.picard_alignment_metrics[s_name]["CATEGORY"] == "PAIR":
                pdata[s_name]["total_reads"] = self.picard_alignment_metrics[s_name]["TOTAL_READS"] / 2
                pdata[s_name]["aligned_reads"] = self.picard_alignment_metrics[s_name]["PF_READS_ALIGNED"] / 2
            else:
                pdata[s_name]["total_reads"] = self.picard_alignment_metrics[s_name]["TOTAL_READS"]
                pdata[s_name]["aligned_reads"] = self.picard_alignment_metrics[s_name]["PF_READS_ALIGNED"]
            pdata[s_name]["unaligned_reads"] = pdata[s_name]["total_reads"] - pdata[s_name]["aligned_reads"]

        keys = [OrderedDict(), OrderedDict()]
        keys[0]["aligned_reads"] = {"name": "Aligned Reads"}
        keys[0]["unaligned_reads"] = {"name": "Unaligned Reads"}
        keys[1]["PF_ALIGNED_BASES"] = {"name": "Aligned Bases"}

        # Config for the plot
        pconfig = {
            "id": "picard_alignment_summary",
            "title": "Picard: Alignment Summary",
            "ylab": "# Reads",
            "data_labels": [
                {
                    "name": "Aligned Reads",
                    "ylab": "# Reads",
                    "cpswitch_counts_label": "Number of Reads",
                },
                {
                    "name": "Aligned Bases",
                    "ylab": "# Bases",
                    "cpswitch_counts_label": "Number of Bases",
                },
            ],
        }

        # The different data sets we want to plot
        self.add_section(
            name="Alignment Summary",
            anchor="picard-alignmentsummary",
            description="Please note that Picard's read counts are divided by two for paired-end data. Total bases (including unaligned) is not provided.",
            plot=bargraph.plot([pdata, self.picard_alignment_metrics], keys, pconfig),
        )

        # Make a bar plot of mean read length
        keys = {"MEAN_READ_LENGTH": {"name": "Mean Read Length"}}
        pconfig = {
            "id": "picard_alignment_readlength_plot",
            "title": "Picard: Mean Read Length",
            "ylab": "Base pairs",
            "cpswitch": False,
        }

        # The different data sets we want to plot
        self.add_section(
            name="Mean read length",
            anchor="picard_alignment_readlength",
            description="The mean read length of the set of reads examined.",
            plot=bargraph.plot(self.picard_alignment_metrics, keys, pconfig),
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_alignment_metrics)



from __future__ import absolute_import

from .mapping_metrics import DragenMappingMetics
from .fragment_length import DragenFragmentLength
from .ploidy_estimation_metrics import DragenPloidyEstimationMetrics
from .vc_metrics import DragenVCMetrics
from .coverage_per_contig import DragenCoveragePerContig
from .coverage_metrics import DragenCoverageMetrics
from .coverage_hist import DragenCoverageHist

import logging

log = logging.getLogger(__name__)


class MultiqcModule(
    DragenMappingMetics,
    DragenFragmentLength,
    DragenPloidyEstimationMetrics,
    DragenVCMetrics,
    DragenCoveragePerContig,
    DragenCoverageMetrics,
    DragenCoverageHist,
):
    """DRAGEN provides a number of differrent pipelines and outputs, including base calling, DNA and RNA alignment,
    post-alignment processing and variant calling, covering virtually all stages of typical NGS data processing.
    However, it can be treated as a fast aligner with additional features on top, as users will unlikely use any
    features without enabling DRAGEN mapping. So we will treat this module as an alignment tool module and
    place it accordingly in the module_order list, in docs, etc.

    The QC metrics DRAGEN generates resemble those of samtools-stats, qualimap, mosdepth, bcftools-stats and alike.
    Whenver possible, the visual output is made similar to those modules.

    Note that this MultiQC module supports some of DRAGEN output but not all. Contributions are welcome!

    The code is structured in a way so every mix-in parses one type of QC file that DRAGEN generates
    (e.g. *.mapping_metrics.csv, *.wgs_fine_hist_normal.csv, etc). If a corresponding file is found, a mix-in adds
    a section into the report.
    """

    def __init__(self):
        super(MultiqcModule, self).__init__(
            name="DRAGEN",
            anchor="DRAGEN",
            target="DRAGEN",
            href="https://www.illumina.com/products/by-type/informatics-products/dragen-bio-it-platform.html",
            info=""" is a Bio-IT Platform that provides ultra-rapid secondary analysis of sequencing data
                     using field-programmable gate array technology (FPGA).""",
            # Can't find a DOI // doi=
        )

        samples_found = set()

        # <output prefix>.vc_metrics.csv
        # a dedicated table and the total number of Variants into the general stats table
        samples_found |= self.add_vc_metrics()

        # <output prefix>.ploidy_estimation_metrics.csv
        # a "Ploidy estimation" column in the general stats table
        samples_found |= self.add_ploidy_estimation_metrics()

        # <output prefix>.(wgs|target_bed)_fine_hist_(tumor|normal)?.csv
        # coverage distribution and cumulative coverage plots
        samples_found |= self.add_coverage_hist()

        # <output prefix>.(wgs|target_bed)_coverage_metrics_(tumor|normal)?.csv
        # general stats table and a dedicated table
        samples_found |= self.add_coverage_metrics()

        # <output prefix>.(wgs|target_bed)_contig_mean_cov_(tumor|normal)?.csv
        # a histogram like in mosdepth, with each chrom as a category on X axis, plus a category
        # for autosomal chromosomes average
        samples_found |= self.add_coverage_per_contig()

        # general stats table, a dedicated table, and a few barplots
        # <output prefix>.mapping_metrics.csv
        samples_found |= self.add_mapping_metrics()

        # a histogram plot
        # <output prefix>.fragment_length_hist.csv
        samples_found |= self.add_fragment_length_hist()

        if len(samples_found) == 0:
            raise UserWarning
        log.info("Found {} reports".format(len(samples_found)))



#!/usr/bin/env python

""" MultiQC module to parse output from Trimmomatic """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name="Trimmomatic",
            anchor="trimmomatic",
            href="http://www.usadellab.org/cms/?page=trimmomatic",
            info="is a flexible read trimming tool for Illumina NGS data.",
            doi="10.1093/bioinformatics/btu170",
        )

        # Parse logs
        self.trimmomatic = dict()
        for f in self.find_log_files("trimmomatic", filehandles=True):
            self.parse_trimmomatic(f)
            self.add_data_source(f)

        # Filter to strip out ignored sample names
        self.trimmomatic = self.ignore_samples(self.trimmomatic)

        if len(self.trimmomatic) == 0:
            raise UserWarning

        log.info("Found {} logs".format(len(self.trimmomatic)))
        self.write_data_file(self.trimmomatic, "multiqc_trimmomatic")

        # Add drop rate to the general stats table
        headers = OrderedDict()
        headers["dropped_pct"] = {
            "title": "% Dropped",
            "description": "% Dropped reads",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "scale": "OrRd",
        }
        self.general_stats_addcols(self.trimmomatic, headers)

        # Make barplot
        self.trimmomatic_barplot()

    def parse_trimmomatic(self, f):
        s_name = None
        if getattr(config, "trimmomatic", {}).get("s_name_filenames", False):
            s_name = f["s_name"]
        for l in f["f"]:
            # Get the sample name
            if s_name is None and "Trimmomatic" in l and "Started with arguments:" in l:
                # Match everything up until the first .fastq or .fq
                match = re.search("Trimmomatic[SP]E: Started with arguments:.+?(?=\.fastq|\.fq)", l)
                if match:
                    # backtrack from the end to the first space
                    s_name = match.group().split()[-1]
                    s_name = self.clean_s_name(s_name, f)
                    if s_name in self.trimmomatic:
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                else:
                    # Try looking on the next line instead, sometimes have a line break (see issue #212)
                    l = next(f["f"])
                    match = re.search(".+?(?=\.fastq|\.fq)", l)
                    if match:
                        # backtrack from the end to the first space
                        s_name = match.group().split()[-1]
                        s_name = self.clean_s_name(s_name, f)
                        if s_name in self.trimmomatic:
                            log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))

            # Get single end stats
            if "Input Reads" in l and s_name is not None:
                match = re.search(
                    "Input Reads: (\d+) Surviving: (\d+) \(([\d\.,]+)%\) Dropped: (\d+) \(([\d\.,]+)%\)", l
                )
                if match:
                    self.trimmomatic[s_name] = dict()
                    self.trimmomatic[s_name]["input_reads"] = float(match.group(1))
                    self.trimmomatic[s_name]["surviving"] = float(match.group(2))
                    self.trimmomatic[s_name]["surviving_pct"] = float(match.group(3).replace(",", "."))
                    self.trimmomatic[s_name]["dropped"] = float(match.group(4))
                    self.trimmomatic[s_name]["dropped_pct"] = float(match.group(5).replace(",", "."))
                    s_name = None

            # Get paired end stats
            if "Input Read Pairs" in l and s_name is not None:
                match = re.search(
                    "Input Read Pairs: (\d+) Both Surviving: (\d+) \(([\d\.,]+)%\) Forward Only Surviving: (\d+) \(([\d\.,]+)%\) Reverse Only Surviving: (\d+) \(([\d\.,]+)%\) Dropped: (\d+) \(([\d\.,]+)%\)",
                    l,
                )
                if match:
                    self.trimmomatic[s_name] = dict()
                    self.trimmomatic[s_name]["input_read_pairs"] = float(match.group(1))
                    self.trimmomatic[s_name]["surviving"] = float(match.group(2))
                    self.trimmomatic[s_name]["surviving_pct"] = float(match.group(3).replace(",", "."))
                    self.trimmomatic[s_name]["forward_only_surviving"] = float(match.group(4))
                    self.trimmomatic[s_name]["forward_only_surviving_pct"] = float(match.group(5).replace(",", "."))
                    self.trimmomatic[s_name]["reverse_only_surviving"] = float(match.group(6))
                    self.trimmomatic[s_name]["reverse_only_surviving_pct"] = float(match.group(7).replace(",", "."))
                    self.trimmomatic[s_name]["dropped"] = float(match.group(8))
                    self.trimmomatic[s_name]["dropped_pct"] = float(match.group(9).replace(",", "."))
                    s_name = None

    def trimmomatic_barplot(self):
        """Make the HighCharts HTML to plot the trimmomatic rates"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys["surviving"] = {"color": "#437bb1", "name": "Surviving Reads"}
        keys["both_surviving"] = {"color": "#f7a35c", "name": "Both Surviving"}
        keys["forward_only_surviving"] = {"color": "#e63491", "name": "Forward Only Surviving"}
        keys["reverse_only_surviving"] = {"color": "#b1084c", "name": "Reverse Only Surviving"}
        keys["dropped"] = {"color": "#7f0000", "name": "Dropped"}

        # Config for the plot
        pconfig = {
            "id": "trimmomatic_plot",
            "title": "Trimmomatic: Surviving Reads",
            "ylab": "# Reads",
            "cpswitch_counts_label": "Number of Reads",
        }

        self.add_section(plot=bargraph.plot(self.trimmomatic, keys, pconfig))



#!/usr/bin/env python

""" MultiQC submodule to parse output from Sentieon InsertSizeMetrics (based
 on the Picard module of the same name """

from collections import OrderedDict
import logging
import os

from multiqc import config
from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Sentieon InsertSizeMetrics reports and parse their data"""

    # Set up vars
    self.sentieon_insertSize_data = dict()
    self.sentieon_insertSize_histogram = dict()
    self.sentieon_insertSize_samplestats = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("sentieon/insertsize", filehandles=True):
        s_name = None
        in_hist = False
        for line in f["f"]:

            # Catch the histogram values
            if s_name is not None and in_hist is True:
                try:
                    sections = line.split("\t")
                    ins = int(sections[0])
                    tot_count = sum([int(x) for x in sections[1:]])
                    self.sentieon_insertSize_histogram[s_name][ins] = tot_count
                    self.sentieon_insertSize_samplestats[s_name]["total_count"] += tot_count
                except ValueError:
                    # Reset in case we have more in this log file
                    s_name = None
                    in_hist = False

            # New log starting
            if s_name is None and "InsertSizeMetricAlgo" in line:
                # Pull sample name from filename
                s_name = os.path.basename(f["s_name"])
                s_name = self.clean_s_name(s_name, f)

            if s_name is not None:
                if "InsertSizeMetricAlgo" in line and "#SentieonCommandLine" in line:
                    if s_name in self.sentieon_insertSize_data:
                        log.debug(
                            "Duplicate sample name found in {}!\
                             Overwriting: {}".format(
                                f["fn"], s_name
                            )
                        )
                    self.add_data_source(f, s_name, section="InsertSizeMetrics")
                    keys = f["f"].readline().strip("\n").split("\t")
                    vals = f["f"].readline().strip("\n").split("\t")
                    self.sentieon_insertSize_samplestats[s_name] = {"total_count": 0, "meansum": 0, "total_pairs": 0}
                    orientation_idx = keys.index("PAIR_ORIENTATION")
                    while len(vals) == len(keys):
                        pair_orientation = vals[orientation_idx]
                        rowkey = "{}_{}".format(s_name, pair_orientation)
                        self.sentieon_insertSize_data[rowkey] = OrderedDict()
                        self.sentieon_insertSize_data[rowkey]["SAMPLE_NAME"] = s_name
                        for i, k in enumerate(keys):
                            try:
                                self.sentieon_insertSize_data[rowkey][k] = float(vals[i])
                            except ValueError:
                                try:
                                    self.sentieon_insertSize_data[rowkey][k] = float(vals[i].replace(",", "."))
                                    log.debug(
                                        "Switching commas for points in '{}':\
                                             {} - {}".format(
                                            f["fn"], vals[i], vals[i].replace(",", ".")
                                        )
                                    )
                                except ValueError:
                                    self.sentieon_insertSize_data[rowkey][k] = vals[i]
                            except IndexError:
                                pass  # missing data
                        # Add to mean sums
                        rp = self.sentieon_insertSize_data[rowkey]["READ_PAIRS"]
                        mis = self.sentieon_insertSize_data[rowkey]["MEAN_INSERT_SIZE"]
                        self.sentieon_insertSize_samplestats[s_name]["meansum"] += rp * mis
                        self.sentieon_insertSize_samplestats[s_name]["total_pairs"] += rp

                        vals = f["f"].readline().strip("\n").split("\t")

                    # Skip lines on to histogram (variable used in later loops)
                    line = f["f"].readline().strip("\n")
                    line = f["f"].readline().strip("\n")

                    self.sentieon_insertSize_histogram[s_name] = OrderedDict()
                    in_hist = True

        for key in list(self.sentieon_insertSize_data.keys()):
            if len(self.sentieon_insertSize_data[key]) == 0:
                self.sentieon_insertSize_data.pop(key, None)
        for s_name in list(self.sentieon_insertSize_histogram.keys()):
            if len(self.sentieon_insertSize_histogram[s_name]) == 0:
                self.sentieon_insertSize_histogram.pop(s_name, None)
                log.debug("Ignoring '{}' histogram as no data parsed".format(s_name))

    # Calculate summed mean values for all read orientations
    for s_name, v in self.sentieon_insertSize_samplestats.items():
        try:
            self.sentieon_insertSize_samplestats[s_name]["summed_mean"] = v["meansum"] / v["total_pairs"]
        except ZeroDivisionError:
            # The mean of zero elements is zero
            self.sentieon_insertSize_samplestats[s_name]["summed_mean"] = 0

    # Calculate summed median values for all read orientations
    for s_name in self.sentieon_insertSize_histogram:
        j = 0
        for idx, c in self.sentieon_insertSize_histogram[s_name].items():
            j += c
            if j > (self.sentieon_insertSize_samplestats[s_name]["total_count"] / 2):
                self.sentieon_insertSize_samplestats[s_name]["summed_median"] = idx
                break

    # Filter to strip out ignored sample names
    self.sentieon_insertSize_data = self.ignore_samples(self.sentieon_insertSize_data)

    if len(self.sentieon_insertSize_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.sentieon_insertSize_data, "multiqc_sentieon_insertSize")

        # Do we have median insert sizes?
        missing_medians = False
        for v in self.sentieon_insertSize_samplestats.values():
            if "summed_median" not in v:
                missing_medians = True

        # Add to general stats table
        self.general_stats_headers["summed_median"] = {
            "title": "Insert Size",
            "description": "Median Insert Size, all read orientations (bp)",
            "min": 0,
            "suffix": " bp",
            "format": "{:,.0f}",
            "scale": "GnBu",
        }
        self.general_stats_headers["summed_mean"] = {
            "title": "Mean Insert Size",
            "description": "Mean Insert Size, all read orientations (bp)",
            "min": 0,
            "suffix": " bp",
            "format": "{:,.0f}",
            "scale": "GnBu",
            "hidden": False if missing_medians else True,
        }
        for s_name in self.sentieon_insertSize_samplestats:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(self.sentieon_insertSize_samplestats[s_name])

        # Section with histogram plot
        if len(self.sentieon_insertSize_histogram) > 0:
            # Make a normalised percentage version of the data
            data_percent = {}
            for s_name, data in self.sentieon_insertSize_histogram.items():
                data_percent[s_name] = OrderedDict()
                total = float(sum(data.values()))
                for k, v in data.items():
                    data_percent[s_name][k] = (v / total) * 100

            # Allow customisation of how smooth the the plot is
            try:
                insertsize_smooth_points = int(config.sentieon_config["insertsize_smooth_points"])
                log.debug(
                    "Custom Sentieon insert size smoothing:\
                     {}".format(
                        insertsize_smooth_points
                    )
                )
            except (AttributeError, KeyError, ValueError):
                insertsize_smooth_points = 500

            # Plot the data and add section
            pconfig = {
                "smooth_points": insertsize_smooth_points,
                "smooth_points_sumcounts": [True, False],
                "id": "sentieon_insert_size",
                "title": "Sentieon: Insert Size",
                "ylab": "Count",
                "xlab": "Insert Size (bp)",
                "xDecimals": False,
                "tt_label": "<b>{point.x} bp</b>: {point.y:.0f}",
                "ymin": 0,
                "data_labels": [
                    {"name": "Counts", "ylab": "Coverage"},
                    {"name": "Percentages", "ylab": "Percentage of Counts"},
                ],
            }
            self.add_section(
                name="Insert Size",
                anchor="sentieon-insertsize",
                description="Plot shows the number of reads at a given insert\
                     size. Reads with different orientations are summed.",
                plot=linegraph.plot([self.sentieon_insertSize_histogram, data_percent], pconfig),
            )

    # Return the number of detected samples to the parent module
    return len(self.sentieon_insertSize_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard RrbsSummaryMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard RrbsSummaryMetrics reports and parse their data"""

    # Set up vars
    self.picard_rrbs_metrics = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/rrbs_metrics", filehandles=True):
        parsed_data = dict()
        s_name = None
        keys = None
        for l in f["f"]:
            # New log starting
            if "CollectRrbsMetrics" in l and "INPUT" in l:
                s_name = None
                keys = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)
                    parsed_data[s_name] = dict()

            if s_name is not None:
                if "RrbsSummaryMetrics" in l and "## METRICS CLASS" in l:
                    keys = f["f"].readline().strip("\n").split("\t")
                elif keys:
                    vals = l.strip("\n").split("\t")
                    if len(vals) == len(keys):
                        for i, k in enumerate(keys):
                            try:
                                parsed_data[s_name][k] = float(vals[i])
                            except ValueError:
                                parsed_data[s_name][k] = vals[i]
                    else:
                        s_name = None
                        keys = None

        # Remove empty dictionaries
        for s_name in list(parsed_data.keys()):
            if len(parsed_data[s_name]) == 0:
                parsed_data.pop(s_name, None)

        # Collect parsed data
        for s_name in parsed_data.keys():
            if s_name in self.picard_rrbs_metrics:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
            self.add_data_source(f, s_name, section="RrbsSummaryMetrics")
            self.picard_rrbs_metrics[s_name] = parsed_data[s_name]

    # Filter to strip out ignored sample names
    self.picard_rrbs_metrics = self.ignore_samples(self.picard_rrbs_metrics)

    if len(self.picard_rrbs_metrics) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_rrbs_metrics, "multiqc_picard_RrbsSummaryMetrics")

        # Add to general stats table
        self.general_stats_headers["PCT_CPG_BASES_CONVERTED"] = {
            "title": "% CpG Methylated",
            "description": "Percentage of times a CpG cytosine was converted",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "format": "{:,.0f}",
            "scale": "RdYlGn-rev",
            "modify": lambda x: 100 - self.multiply_hundred(x),
        }
        self.general_stats_headers["PCT_NON_CPG_BASES_CONVERTED"] = {
            "title": "% Non-CpG Methylated",
            "description": "Percentage of times a non-CpG cytosine was converted",
            "max": 100,
            "min": 0,
            "suffix": "%",
            "format": "{:,.0f}",
            "scale": "RdYlGn",
            "modify": lambda x: 100 - self.multiply_hundred(x),
        }
        self.general_stats_headers["MEDIAN_CPG_COVERAGE"] = {
            "title": "Median CpG Cov",
            "description": "Median coverage of CpG sites",
            "min": 0,
        }
        for s_name in self.picard_rrbs_metrics:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(self.picard_rrbs_metrics[s_name])

        # Make the bar plot of converted bases
        pdata_cpg = dict()
        pdata_noncpg = dict()
        for s_name in self.picard_rrbs_metrics.keys():
            pdata_cpg[s_name] = dict()
            pdata_cpg[s_name]["converted"] = self.picard_rrbs_metrics[s_name]["CPG_BASES_CONVERTED"]
            pdata_cpg[s_name]["not_converted"] = (
                self.picard_rrbs_metrics[s_name]["CPG_BASES_SEEN"]
                - self.picard_rrbs_metrics[s_name]["CPG_BASES_CONVERTED"]
            )
            pdata_noncpg[s_name] = dict()
            pdata_noncpg[s_name]["converted"] = self.picard_rrbs_metrics[s_name]["NON_CPG_BASES"]
            pdata_noncpg[s_name]["not_converted"] = (
                self.picard_rrbs_metrics[s_name]["NON_CPG_BASES"]
                - self.picard_rrbs_metrics[s_name]["NON_CPG_CONVERTED_BASES"]
            )

        keys = OrderedDict()
        keys["not_converted"] = {"name": "Unconverted Bases (Methylated)"}
        keys["converted"] = {"name": "Converted Bases (Unmethylated)"}

        # Config for the plot
        pconfig = {
            "id": "picard_rrbs_converted_bases_plot",
            "title": "Picard: Converted Bases",
            "ylab": "# CpG Bases",
            "cpswitch_counts_label": "Number of Bases",
            "data_labels": [{"name": "CpG", "ylab": "# CpG Bases"}, {"name": "Non-CpG", "ylab": "# Non-CpG Bases"}],
        }

        self.add_section(
            name="RRBS Converted Bases",
            anchor="picard-rrbssummary-convertedbases",
            plot=bargraph.plot([pdata_cpg, pdata_noncpg], [keys, keys], pconfig),
        )

        # Make the bar plot of processed reads
        pdata = dict()
        for s_name in self.picard_rrbs_metrics.keys():
            pdata[s_name] = dict()
            pdata[s_name]["with_no_cpg"] = self.picard_rrbs_metrics[s_name]["READS_WITH_NO_CPG"]
            pdata[s_name]["ignored_short"] = self.picard_rrbs_metrics[s_name]["READS_IGNORED_SHORT"]
            pdata[s_name]["ignored_mismatches"] = self.picard_rrbs_metrics[s_name]["READS_IGNORED_MISMATCHES"]
            pdata[s_name]["not_ignored"] = (
                self.picard_rrbs_metrics[s_name]["READS_ALIGNED"]
                - pdata[s_name]["with_no_cpg"]
                - pdata[s_name]["ignored_short"]
                - pdata[s_name]["ignored_mismatches"]
            )

        keys = OrderedDict()
        keys["not_ignored"] = {"name": "Utilised reads"}
        keys["with_no_cpg"] = {"name": "Ignored (no CpG sites)"}
        keys["ignored_short"] = {"name": "Ignored (too short)"}
        keys["ignored_mismatches"] = {"name": "Ignored (exceeded mismatch threshold)"}

        # Config for the plot
        pconfig = {
            "id": "picard_rrbs_ignored_reads_plot",
            "title": "Picard: RRBS Read Counts",
            "ylab": "# Reads",
            "cpswitch_counts_label": "Number of Reads",
        }

        self.add_section(
            name="RRBS Read Counts", anchor="picard-rrbssummary-readcounts", plot=bargraph.plot(pdata, keys, pconfig)
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_rrbs_metrics)



#!/usr/bin/env python

""" MultiQC submodule to parse output from deepTools estimateReadFiltering """

import logging
from collections import OrderedDict

from multiqc.plots import table

# Initialise the logger
log = logging.getLogger(__name__)


class estimateReadFilteringMixin:
    def parse_estimateReadFiltering(self):
        """Find estimateReadFiltering output. Only the output from --table is supported."""
        self.deeptools_estimateReadFiltering = dict()
        for f in self.find_log_files("deeptools/estimateReadFiltering"):
            parsed_data = self.parseEstimateReadFilteringFile(f)
            for k, v in parsed_data.items():
                if k in self.deeptools_estimateReadFiltering:
                    log.warning("Replacing duplicate sample {}.".format(k))
                self.deeptools_estimateReadFiltering[k] = v

            if len(parsed_data) > 0:
                self.add_data_source(f, section="estimateReadFiltering")

        self.deeptools_estimateReadFiltering = self.ignore_samples(self.deeptools_estimateReadFiltering)

        if len(self.deeptools_estimateReadFiltering) > 0:
            # Write data to file
            self.write_data_file(self.deeptools_estimateReadFiltering, "deeptools_read_filtering")

            header = OrderedDict()
            header["M Entries"] = {"title": "M entries", "description": "Number of entries in the file (millions)"}
            header["pct_Aligned"] = {
                "title": "% Aligned",
                "description": "Percent of aligned entries",
                "scale": "YlGn",
                "min": 0,
                "max": 100,
            }
            header["pct_Filtered"] = {
                "title": "% Tot. Filtered",
                "description": "Percent of alignment that would be filtered for any reason.",
                "scale": "OrRd",
                "min": 0,
                "max": 100,
            }
            header["pct_Blacklisted"] = {
                "title": "% Blacklisted",
                "description": "Percent of alignments falling (at least partially) inside a blacklisted region",
                "scale": "YlOrRd",
                "min": 0,
                "max": 100,
            }
            header["pct_MAPQ"] = {
                "title": "% MAPQ",
                "description": "Percent of alignments having MAPQ scores below the specified threshold",
                "scale": "YlOrBn",
                "min": 0,
                "max": 100,
            }
            header["pct_Missing_Flags"] = {
                "title": "% Missing Flags",
                "description": "Percent of alignments lacking at least on flag specified by --samFlagInclude",
                "scale": "PuRd",
                "min": 0,
                "max": 100,
            }
            header["pct_Forbidden_Flags"] = {
                "title": "% Forbidden Flags",
                "description": "Percent of alignments having at least one flag specified by --samFlagExclude",
                "scale": "OrRd",
                "min": 0,
                "max": 100,
            }
            header["pct_deepTools_Dupes"] = {
                "title": "% deepTools Dupes",
                "description": "Percent of alignments marked by deepTools as being duplicates",
                "scale": "PuRd",
                "min": 0,
                "max": 100,
            }
            header["pct_Duplication"] = {
                "title": "% Duplication",
                "description": "Percent of alignments originally marked as being duplicates",
                "scale": "OrRd",
                "min": 0,
                "max": 100,
            }
            header["pct_Singletons"] = {
                "title": "% Singletons",
                "description": "Percent of alignments that are singletons (i.e., paired-end reads where the mates don't align as a pair",
                "scale": "OrRd",
                "min": 0,
                "max": 100,
            }
            header["pct_Strand_Filtered"] = {
                "title": "% Strand Filtered",
                "description": "Percent of alignments arising from the wrong strand",
                "scale": "OrRd",
                "min": 0,
                "max": 100,
            }

            tdata = dict()
            for k, v in self.deeptools_estimateReadFiltering.items():
                tdata[k] = {
                    "M Entries": v["total"] / 1000000.0,
                    "pct_Aligned": 100.0 * v["mapped"] / float(v["total"]),
                    "pct_Filtered": 100.0 * v["filtered"] / float(v["total"]),
                    "pct_Blacklisted": 100.0 * v["blacklisted"] / float(v["total"]),
                    "pct_Below_MAPQ": 100.0 * v["mapq"] / float(v["total"]),
                    "pct_Missing_Flags": 100.0 * v["required flags"] / float(v["total"]),
                    "pct_Forbidden_Flags": 100.0 * v["excluded flags"] / float(v["total"]),
                    "pct_deepTools_Dupes": 100.0 * v["internal dupes"] / float(v["total"]),
                    "pct_Duplication": 100.0 * v["dupes"] / float(v["total"]),
                    "pct_Singletons": 100.0 * v["singletons"] / float(v["total"]),
                    "pct_Strand_Filtered": 100.0 * v["strand"] / float(v["total"]),
                }

            config = {"namespace": "deepTools bamPEFragmentSize"}
            self.add_section(
                name="Filtering metrics",
                anchor="estimateReadFiltering",
                description="Estimated percentages of alignments filtered independently for each setting in `estimateReadFiltering`",
                plot=table.plot(tdata, header, config),
            )

        return len(self.deeptools_estimateReadFiltering)

    def parseEstimateReadFilteringFile(self, f):
        d = {}
        firstLine = True
        for line in f["f"].splitlines():
            if firstLine:
                firstLine = False
                continue
            cols = line.strip().split("\t")

            if len(cols) != 12:
                # This is not really the output from estimateReadFiltering!
                log.warning(
                    "{} was initially flagged as the tabular output from estimateReadFiltering, but that seems to not be the case. Skipping...".format(
                        f["fn"]
                    )
                )
                return dict()

            s_name = self.clean_s_name(cols[0], f)
            if s_name in d:
                log.debug("Replacing duplicate sample {}.".format(s_name))
            d[s_name] = dict()

            try:
                d[s_name]["total"] = self._int(cols[1])
                d[s_name]["mapped"] = self._int(cols[2])
                d[s_name]["blacklisted"] = self._int(cols[3])
                d[s_name]["filtered"] = float(cols[4])
                d[s_name]["mapq"] = float(cols[5])
                d[s_name]["required flags"] = float(cols[6])
                d[s_name]["excluded flags"] = float(cols[7])
                d[s_name]["internal dupes"] = float(cols[8])
                d[s_name]["dupes"] = float(cols[9])
                d[s_name]["singletons"] = float(cols[10])
                d[s_name]["strand"] = float(cols[11])
            except:
                # Obviously this isn't really the output from estimateReadFiltering
                log.warning(
                    "{} was initially flagged as the output from estimateReadFiltering, but that seems to not be the case. Skipping...".format(
                        f["fn"]
                    )
                )
                return dict()
        return d



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard OxoGMetrics """

import logging
import os
import re

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard OxoGMetrics reports and parse their data"""

    # Set up vars
    self.picard_OxoGMetrics_data = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/oxogmetrics", filehandles=True):
        # We use lists to make sure that we don't overwrite when no data will be parsed
        parsed_data = list()
        sample_names = list()
        s_files = list()
        s_name = None
        keys = None
        for l in f["f"]:
            # New log starting
            if ("CollectOxoGMetrics" in l or "ConvertSequencingArtifactToOxoG" in l) and "INPUT" in l:
                s_name = None
                keys = None
                context_col = None

                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:_BASE)?(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)
                    parsed_data.append(dict())
                    sample_names.append(s_name)
                    s_files.append(f)

            if s_name is not None:
                if "CollectOxoGMetrics$CpcgMetrics" in l and "## METRICS CLASS" in l:
                    keys = f["f"].readline().strip("\n").split("\t")
                    context_col = keys.index("CONTEXT")
                elif keys:
                    vals = l.strip("\n").split("\t")
                    if len(vals) == len(keys) and context_col is not None:
                        context = vals[context_col]
                        parsed_data[-1][context] = dict()
                        for i, k in enumerate(keys):
                            k = k.strip()
                            try:
                                parsed_data[-1][context][k] = float(vals[i])
                            except ValueError:
                                vals[i] = vals[i].strip()
                                parsed_data[-1][context][k] = vals[i]
                    else:
                        s_name = None
                        keys = None

        # Remove empty dictionaries
        for idx, s_name in enumerate(sample_names):
            if len(parsed_data[idx]) > 0:
                if s_name in self.picard_OxoGMetrics_data:
                    log.debug("Duplicate sample name found in {}! Overwriting: {}".format(s_files[idx], s_name))
                self.add_data_source(s_files[idx], s_name, section="OxoGMetrics")
                self.picard_OxoGMetrics_data[s_name] = parsed_data[idx]

    # Filter to strip out ignored sample names
    self.picard_OxoGMetrics_data = self.ignore_samples(self.picard_OxoGMetrics_data)

    if len(self.picard_OxoGMetrics_data) > 0:

        # Write parsed data to a file
        # Collapse into 2D structure with sample_context keys
        print_data = {
            "{}_{}".format(s, c): v
            for s in self.picard_OxoGMetrics_data.keys()
            for c, v in self.picard_OxoGMetrics_data[s].items()
        }
        self.write_data_file(print_data, "multiqc_picard_OxoGMetrics")

        # Add to general stats table
        data = dict()
        for s_name in self.picard_OxoGMetrics_data:
            data[s_name] = dict()
            try:
                data[s_name]["CCG_OXIDATION_ERROR_RATE"] = self.picard_OxoGMetrics_data[s_name]["CCG"][
                    "OXIDATION_ERROR_RATE"
                ]
            except KeyError:
                log.warning("Couldn't find picard CCG oxidation error rate for {}".format(s_name))

        self.general_stats_headers["CCG_OXIDATION_ERROR_RATE"] = {
            "title": "CCG Oxidation",
            "description": "CCG-CAG Oxidation Error Rate",
            "max": 1,
            "min": 0,
            "suffix": "%",
            "format": "{:,.0f}",
            "scale": "RdYlGn-rev",
            "modify": lambda x: self.multiply_hundred(x),
        }
        for s_name in data:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update(data[s_name])

    # Return the number of detected samples to the parent module
    return len(self.picard_OxoGMetrics_data)



#!/usr/bin/env python
"""
MultiQC is a tool to aggregate bioinformatics results across many samples into a single report. It is written in Python and contains modules for a large number of common bioinformatics tools.

You can install MultiQC from PyPI as follows::

    pip install multiqc

Then it's just a case of going to your analysis directory and running the script::

    multiqc .

MultiQC will scan the specified directory (:code:`'.'` is the current dir) and produce a report detailing whatever it finds.

The report is created in :code:`multiqc_report.html` by default. Tab-delimited data files are created in :code:`multiqc_data/` to give easy access for downstream processing.

For more detailed instructions, run :code:`multiqc -h`

See the MultiQC website for documentation and tutorial videos: http://multiqc.info

MultiQC was written by Phil Ewels (http://phil.ewels.co.uk) at SciLifeLab Sweden (http://www.scilifelab.se)
"""

from setuptools import setup, find_packages
import sys

version = "1.12"
dl_version = "master" if "dev" in version else "v{}".format(version)

print(
    """-----------------------------------
 Installing MultiQC version {}
-----------------------------------

""".format(
        version
    )
)

# Set version requirements according to what version of Python we're running
networkx_version = ">=2.5.1"  # Needed for Python 3.9 support
numpy_version = ""
matplotlib_version = ">=2.1.1"
jinja2_version = ">=2.9"
markdown_version = ""
if sys.version_info[0:2] < (3, 6):
    # Lots of tools have dropped Python 3 support, so limit their versions
    matplotlib_version += ",<3.0.0"
    numpy_version = "<1.17"
    networkx_version = "<2.3"
    jinja2_version += ",<3.0"
    markdown_version = "<3.2"

install_requires = [
    "matplotlib" + matplotlib_version,
    "networkx" + networkx_version,
    "numpy" + numpy_version,
    "click",
    "coloredlogs",
    "future>0.14.0",
    "jinja2" + jinja2_version,
    "lzstring",
    "markdown" + markdown_version,
    "pyyaml>=4",
    "requests",
    "rich>=10",
    "simplejson",
    "spectra>=0.0.10",
]

setup(
    name="multiqc",
    version=version,
    author="Phil Ewels",
    author_email="phil.ewels@scilifelab.se",
    description="Create aggregate bioinformatics analysis reports across many samples and tools",
    long_description=__doc__,
    keywords=["bioinformatics", "biology", "sequencing", "NGS", "next generation sequencing", "quality control"],
    url="http://multiqc.info",
    download_url="https://github.com/ewels/MultiQC/tarball/{}".format(dl_version),
    license="GPLv3",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            "multiqc=multiqc.__main__:multiqc",
        ],
        "multiqc.modules.v1": [
            "adapterRemoval = multiqc.modules.adapterRemoval:MultiqcModule",
            "afterqc = multiqc.modules.afterqc:MultiqcModule",
            "bamtools = multiqc.modules.bamtools:MultiqcModule",
            "bbmap = multiqc.modules.bbmap:MultiqcModule",
            "bcftools = multiqc.modules.bcftools:MultiqcModule",
            "bcl2fastq = multiqc.modules.bcl2fastq:MultiqcModule",
            "bclconvert = multiqc.modules.bclconvert:MultiqcModule",
            "biobambam2 = multiqc.modules.biobambam2:MultiqcModule",
            "biobloomtools = multiqc.modules.biobloomtools:MultiqcModule",
            "biscuit = multiqc.modules.biscuit:MultiqcModule",
            "bismark = multiqc.modules.bismark:MultiqcModule",
            "bowtie1 = multiqc.modules.bowtie1:MultiqcModule",
            "bowtie2 = multiqc.modules.bowtie2:MultiqcModule",
            "busco = multiqc.modules.busco:MultiqcModule",
            "bustools = multiqc.modules.bustools:MultiqcModule",
            "ccs = multiqc.modules.ccs:MultiqcModule",
            "checkqc = multiqc.modules.checkqc:MultiqcModule",
            "clipandmerge = multiqc.modules.clipandmerge:MultiqcModule",
            "clusterflow = multiqc.modules.clusterflow:MultiqcModule",
            "conpair = multiqc.modules.conpair:MultiqcModule",
            "custom_content = multiqc.modules.custom_content:custom_module_classes",  # special case
            "cutadapt = multiqc.modules.cutadapt:MultiqcModule",
            "damageprofiler = multiqc.modules.damageprofiler:MultiqcModule",
            "dedup = multiqc.modules.dedup:MultiqcModule",
            "deeptools = multiqc.modules.deeptools:MultiqcModule",
            "disambiguate = multiqc.modules.disambiguate:MultiqcModule",
            "dragen = multiqc.modules.dragen:MultiqcModule",
            "eigenstratdatabasetools = multiqc.modules.eigenstratdatabasetools:MultiqcModule",
            "fastp = multiqc.modules.fastp:MultiqcModule",
            "fastq_screen = multiqc.modules.fastq_screen:MultiqcModule",
            "fastqc = multiqc.modules.fastqc:MultiqcModule",
            "featureCounts = multiqc.modules.featureCounts:MultiqcModule",
            "fgbio = multiqc.modules.fgbio:MultiqcModule",
            "flash = multiqc.modules.flash:MultiqcModule",
            "flexbar = multiqc.modules.flexbar:MultiqcModule",
            "gffcompare = multiqc.modules.gffcompare:MultiqcModule",
            "gatk = multiqc.modules.gatk:MultiqcModule",
            "goleft_indexcov = multiqc.modules.goleft_indexcov:MultiqcModule",
            "happy = multiqc.modules.happy:MultiqcModule",
            "hicexplorer = multiqc.modules.hicexplorer:MultiqcModule",
            "hicpro = multiqc.modules.hicpro:MultiqcModule",
            "hicup = multiqc.modules.hicup:MultiqcModule",
            "hisat2 = multiqc.modules.hisat2:MultiqcModule",
            "homer = multiqc.modules.homer:MultiqcModule",
            "hops = multiqc.modules.hops:MultiqcModule",
            "htseq = multiqc.modules.htseq:MultiqcModule",
            "interop = multiqc.modules.interop:MultiqcModule",
            "ivar = multiqc.modules.ivar:MultiqcModule",
            "jcvi = multiqc.modules.jcvi:MultiqcModule",
            "jellyfish = multiqc.modules.jellyfish:MultiqcModule",
            "kaiju = multiqc.modules.kaiju:MultiqcModule",
            "kallisto = multiqc.modules.kallisto:MultiqcModule",
            "kat = multiqc.modules.kat:MultiqcModule",
            "kraken = multiqc.modules.kraken:MultiqcModule",
            "leehom = multiqc.modules.leehom:MultiqcModule",
            "lima = multiqc.modules.lima:MultiqcModule",
            "longranger = multiqc.modules.longranger:MultiqcModule",
            "macs2 = multiqc.modules.macs2:MultiqcModule",
            "malt = multiqc.modules.malt:MultiqcModule",
            "methylQA = multiqc.modules.methylQA:MultiqcModule",
            "minionqc = multiqc.modules.minionqc:MultiqcModule",
            "mirtop = multiqc.modules.mirtop:MultiqcModule",
            "mirtrace = multiqc.modules.mirtrace:MultiqcModule",
            "mosdepth = multiqc.modules.mosdepth:MultiqcModule",
            "mtnucratio = multiqc.modules.mtnucratio:MultiqcModule",
            "multivcfanalyzer = multiqc.modules.multivcfanalyzer:MultiqcModule",
            "nanostat = multiqc.modules.nanostat:MultiqcModule",
            "ngsderive = multiqc.modules.ngsderive:MultiqcModule",
            "odgi = multiqc.modules.odgi:MultiqcModule",
            "optitype = multiqc.modules.optitype:MultiqcModule",
            "pangolin = multiqc.modules.pangolin:MultiqcModule",
            "pbmarkdup = multiqc.modules.pbmarkdup:MultiqcModule",
            "peddy = multiqc.modules.peddy:MultiqcModule",
            "phantompeakqualtools = multiqc.modules.phantompeakqualtools:MultiqcModule",
            "picard = multiqc.modules.picard:MultiqcModule",
            "preseq = multiqc.modules.preseq:MultiqcModule",
            "prokka = multiqc.modules.prokka:MultiqcModule",
            "purple = multiqc.modules.purple:MultiqcModule",
            "pychopper = multiqc.modules.pychopper:MultiqcModule",
            "pycoqc = multiqc.modules.pycoqc:MultiqcModule",
            "qc3C = multiqc.modules.qc3C:MultiqcModule",
            "qorts = multiqc.modules.qorts:MultiqcModule",
            "qualimap = multiqc.modules.qualimap:MultiqcModule",
            "quast = multiqc.modules.quast:MultiqcModule",
            "rna_seqc = multiqc.modules.rna_seqc:MultiqcModule",
            "rockhopper = multiqc.modules.rockhopper:MultiqcModule",
            "rsem = multiqc.modules.rsem:MultiqcModule",
            "rseqc = multiqc.modules.rseqc:MultiqcModule",
            "salmon = multiqc.modules.salmon:MultiqcModule",
            "sambamba = multiqc.modules.sambamba:MultiqcModule",
            "samblaster = multiqc.modules.samblaster:MultiqcModule",
            "samtools = multiqc.modules.samtools:MultiqcModule",
            "sargasso = multiqc.modules.sargasso:MultiqcModule",
            "sentieon = multiqc.modules.sentieon:MultiqcModule",
            "seqyclean = multiqc.modules.seqyclean:MultiqcModule",
            "sexdeterrmine = multiqc.modules.sexdeterrmine:MultiqcModule",
            "sickle = multiqc.modules.sickle:MultiqcModule",
            "skewer = multiqc.modules.skewer:MultiqcModule",
            "slamdunk = multiqc.modules.slamdunk:MultiqcModule",
            "snippy = multiqc.modules.snippy:MultiqcModule",
            "snpeff = multiqc.modules.snpeff:MultiqcModule",
            "snpsplit = multiqc.modules.snpsplit:MultiqcModule",
            "somalier = multiqc.modules.somalier:MultiqcModule",
            "sortmerna = multiqc.modules.sortmerna:MultiqcModule",
            "stacks = multiqc.modules.stacks:MultiqcModule",
            "star = multiqc.modules.star:MultiqcModule",
            "supernova = multiqc.modules.supernova:MultiqcModule",
            "theta2 = multiqc.modules.theta2:MultiqcModule",
            "tophat = multiqc.modules.tophat:MultiqcModule",
            "trimmomatic = multiqc.modules.trimmomatic:MultiqcModule",
            "varscan2 = multiqc.modules.varscan2:MultiqcModule",
            "vcftools = multiqc.modules.vcftools:MultiqcModule",
            "vep = multiqc.modules.vep:MultiqcModule",
            "verifybamid = multiqc.modules.verifybamid:MultiqcModule",
            "whatshap = multiqc.modules.whatshap:MultiqcModule",
        ],
        "multiqc.templates.v1": [
            "default = multiqc.templates.default",
            "default_dev = multiqc.templates.default_dev",
            "sections = multiqc.templates.sections",
            "simple = multiqc.templates.simple",
            "gathered = multiqc.templates.gathered",
            "geo = multiqc.templates.geo",
        ],
        ## See https://multiqc.info/docs/#multiqc-plugins for documentation
        #       'multiqc.cli_options.v1': [
        #           'my-new-option = myplugin.cli:new_option'
        #       ],
        #       'multiqc.hooks.v1': [
        #           'before_config = myplugin.hooks:before_config',
        #           'config_loaded = myplugin.hooks:config_loaded',
        #           'execution_start = myplugin.hooks:execution_start',
        #           'before_modules = myplugin.hooks:before_modules',
        #           'after_modules = myplugin.hooks:after_modules',
        #           'before_report_generation = myplugin.hooks:before_report_generation',
        #           'execution_finish = myplugin.hooks:execution_finish',
        #       ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Programming Language :: JavaScript",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Visualization",
    ],
)

print(
    """
--------------------------------
 MultiQC installation complete!
--------------------------------
For help in running MultiQC, please see the documentation available
at http://multiqc.info or run: multiqc --help
"""
)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard GcBiasMetrics """

import logging
import os
import re

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard GcBiasMetrics reports and parse their data"""

    # Set up vars
    self.picard_GCbias_data = dict()
    self.picard_GCbiasSummary_data = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/gcbias", filehandles=True):
        s_name = None
        gc_col = None
        cov_col = None
        for l in f["f"]:
            # New log starting
            if "GcBiasMetrics" in l and "INPUT" in l:
                s_name = None

                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip("[]"))
                    s_name = self.clean_s_name(s_name, f)

            if s_name is not None:
                if gc_col is not None and cov_col is not None:
                    try:
                        # Note that GC isn't always the first column.
                        s = l.strip("\n").split("\t")
                        self.picard_GCbias_data[s_name][int(s[gc_col])] = float(s[cov_col])
                    except IndexError:
                        s_name = None
                        gc_col = None
                        cov_col = None

                if "GcBiasDetailMetrics" in l and "## METRICS CLASS" in l:
                    if s_name in self.picard_GCbias_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
                    self.add_data_source(f, s_name, section="GcBiasDetailMetrics")
                    self.picard_GCbias_data[s_name] = dict()
                    # Get header - find columns with the data we want
                    l = f["f"].readline()
                    s = l.strip("\n").split("\t")
                    gc_col = s.index("GC")
                    cov_col = s.index("NORMALIZED_COVERAGE")

                if "GcBiasSummaryMetrics" in l and "## METRICS CLASS" in l:
                    if s_name in self.picard_GCbiasSummary_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], s_name))
                    self.add_data_source(f, s_name, section="GcBiasSummaryMetrics")
                    self.picard_GCbiasSummary_data[s_name] = dict()

                    keys = f["f"].readline().rstrip("\n").split("\t")
                    vals = f["f"].readline().rstrip("\n").split("\t")
                    for i, k in enumerate(keys):
                        try:
                            self.picard_GCbiasSummary_data[s_name][k] = float(vals[i])
                        except ValueError:
                            self.picard_GCbiasSummary_data[s_name][k] = vals[i]

        for s_name in list(self.picard_GCbias_data.keys()):
            if len(self.picard_GCbias_data[s_name]) == 0:
                self.picard_GCbias_data.pop(s_name, None)
                log.debug("Removing {} as no data parsed".format(s_name))

        for s_name in list(self.picard_GCbiasSummary_data.keys()):
            if len(self.picard_GCbiasSummary_data[s_name]) == 0:
                self.picard_GCbiasSummary_data.pop(s_name, None)
                log.debug("Removing {} as no data parsed".format(s_name))

    # Filter to strip out ignored sample names
    self.picard_GCbias_data = self.ignore_samples(self.picard_GCbias_data)

    if len(self.picard_GCbias_data) > 0:

        # Plot the graph

        pconfig = {
            "id": "picard_gcbias_plot",
            "title": "Picard: GC Coverage Bias",
            "ylab": "Normalized Coverage",
            "xlab": "% GC",
            "xmin": 0,
            "xmax": 100,
            "xDecimals": False,
            "ymin": 0,
            "yCeiling": 10,
            "tt_label": "<b>{point.x} %GC</b>: {point.y:.2f}",
            "yPlotLines": [
                {"value": 1, "color": "#999999", "width": 2, "dashStyle": "LongDash"},
            ],
        }
        self.add_section(
            name="GC Coverage Bias",
            anchor="picard-gcbias",
            description="This plot shows bias in coverage across regions of the genome with varying GC content."
            " A perfect library would be a flat line at <code>y = 1</code>.",
            plot=linegraph.plot(self.picard_GCbias_data, pconfig),
        )

    if len(self.picard_GCbiasSummary_data) > 0:
        # Write parsed summary data to a file
        self.write_data_file(self.picard_GCbiasSummary_data, "multiqc_picard_gcbias")

    # Return the number of detected samples to the parent module
    return len(self.picard_GCbias_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard CollectIlluminaBasecallingMetrics """

from collections import OrderedDict
import logging

from multiqc.plots import table, bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """Find Picard CollectIlluminaBasecallingMetrics reports and parse their data"""

    # Set up vars
    self.picard_basecalling_metrics = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/collectilluminabasecallingmetrics", filehandles=True):
        self.add_data_source(f, section="CollectIlluminaBasecallingMetrics")

        keys = None
        for line in f["f"]:
            if "IlluminaBasecallingMetrics" and "## METRICS CLASS" in line:
                keys = f["f"].readline().strip("\n").split("\t")
            elif keys:
                vals = line.strip("\n").split("\t")
                if len(vals) == len(keys):
                    data = dict(zip(keys, vals))
                    if not data["MOLECULAR_BARCODE_SEQUENCE_1"]:
                        data.pop("MOLECULAR_BARCODE_SEQUENCE_1")
                        data.pop("MOLECULAR_BARCODE_NAME")
                        self.picard_basecalling_metrics[data["LANE"]] = data

    # Filter to strip out ignored sample names
    self.picard_basecalling_metrics = self.ignore_samples(self.picard_basecalling_metrics)

    if len(self.picard_basecalling_metrics) > 0:
        # Write parsed data to a file
        self.write_data_file(self.picard_basecalling_metrics, "multiqc_picard_IlluminaBasecallingMetrics")

        self.add_section(
            name="Basecalling Metrics",
            anchor="picard-illuminabasecallingmetrics",
            description="Quality control metrics for each lane of an Illumina flowcell.",
            helptext="""
            For full details, please see the [Picard Documentation](http://broadinstitute.github.io/picard/picard-metric-definitions.html#IlluminaBasecallingMetrics).

            * `PF_BASES` / `NPF_BASES` :  The total number of passing-filter / not-passing-filter bases assigned to the index.
            * `PF_READS` / `NPF_READS` :  The total number of passing-filter / not-passing-filter reads assigned to the index.
            * `PF_CLUSTERS` / `NPF_CLUSTERS` :  The total number of passing-filter / not-passing-filter clusters assigned to the index.

            `NPF` stands for _"not passing filter"_ and is calculated by subtracting the `PF_` metric from the `TOTAL_` Picard metrics.
            """,
            plot=lane_metrics_plot(self.picard_basecalling_metrics),
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_basecalling_metrics)


def lane_metrics_table(data):
    headers = OrderedDict()
    headers["TOTAL_BASES"] = {"title": "Total Bases"}
    headers["PF_BASES"] = {"title": "Passing Filter Bases"}
    headers["TOTAL_READS"] = {"title": "Total Reads"}
    headers["PF_READS"] = {"title": "Passing Filter Reads"}
    headers["TOTAL_CLUSTERS"] = {"title": "Total Cluster"}
    headers["PF_CLUSTERS"] = {"title": "Passing Filter Clusters"}

    table_config = {
        "id": "picard-illumina-basecalling-metrics-table",
        "namespace": "Picard",
        "table_title": "Picard Illumina Base Calling Metrics",
    }
    tdata = {}
    for lane_number, lane in data.items():
        tdata[f"L{lane_number}"] = lane
    return table.plot(tdata, headers, table_config)


def lane_metrics_plot(data):
    plot_config = {
        "id": "plot-picard-illuminabasecallingmetrics",
        "title": "Picard: Illumina Basecalling Metrics",
        "ylab": "Lane",
        "data_labels": [
            {"name": "Bases", "ylab": "Number of Bases"},
            {"name": "Reads", "ylab": "Number of Reads"},
            {"name": "Clusters", "ylab": "Number of Clusters"},
        ],
    }

    plot_cats = [OrderedDict(), OrderedDict(), OrderedDict()]
    plot_cats[0]["PF_BASES"] = {"title": "Passing Filter Bases"}
    plot_cats[0]["NPF_BASES"] = {"title": "Non Passing Filter Bases"}
    plot_cats[1]["PF_READS"] = {"title": "Passing Filter Reads"}
    plot_cats[1]["NPF_READS"] = {"title": "Non Passing Filter Reads"}
    plot_cats[2]["PF_CLUSTERS"] = {"title": "Passing Filter Clusters"}
    plot_cats[2]["NPF_CLUSTERS"] = {"title": "Non Passing Filter Clusters"}
    tdata = {}
    for lane_number, lane in data.items():
        lane["NPF_BASES"] = int(lane["TOTAL_BASES"]) - int(lane["PF_BASES"])
        lane["NPF_READS"] = int(lane["TOTAL_READS"]) - int(lane["PF_READS"])
        lane["NPF_CLUSTERS"] = int(lane["TOTAL_CLUSTERS"]) - int(lane["PF_CLUSTERS"])
        tdata[f"L{lane_number}"] = lane
    return bargraph.plot([tdata, tdata, tdata], plot_cats, plot_config)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard QualityYieldMetrics """

from collections import OrderedDict, defaultdict
import logging
import os
import re

from multiqc import config
from multiqc.plots import table, linegraph
from .util import read_sample_name

# Initialise the logger
log = logging.getLogger(__name__)

DESC = OrderedDict(
    [
        ("TOTAL_READS", "The total number of reads in the input file"),
        ("PF_READS", "The number of reads that are PF - pass filter"),
        ("READ_LENGTH", "The average read length of all the reads (will be fixed for a lane)"),
        ("TOTAL_BASES", "The total number of bases in all reads"),
        ("PF_BASES", "The total number of bases in all PF reads"),
        ("Q20_BASES", "The number of bases in all reads that achieve quality score 20 or higher"),
        ("PF_Q20_BASES", "The number of bases in PF reads that achieve quality score 20 or higher"),
        ("Q30_BASES", "The number of bases in all reads that achieve quality score 30 or higher"),
        ("PF_Q30_BASES", "The number of bases in PF reads that achieve quality score 30 or higher"),
        ("Q20_EQUIVALENT_YIELD", "The sum of quality scores of all bases divided by 20"),
        ("PF_Q20_EQUIVALENT_YIELD", "The sum of quality scores of all bases in PF reads divided by 20"),
    ]
)


def parse_reports(self):
    """Find Picard QualityYieldMetrics reports and parse their data"""

    # Set up vars
    all_data = OrderedDict()
    header = list(DESC.keys())

    # Go through logs and find Metrics
    for f in self.find_log_files("picard/quality_yield_metrics", filehandles=True):
        self.add_data_source(f, section="QualityYieldMetrics")

        parsed_data = dict()
        s_name = None
        keys = None
        commadecimal = None

        lines = iter(f["f"])

        clean_fn = lambda n: self.clean_s_name(n, f)
        s_name = read_sample_name(lines, clean_fn, "CollectQualityYieldMetrics")

        if s_name is None:
            continue

        sample_data = dict()
        try:
            # skip to the histogram
            line = next(lines)
            while not line.startswith("## METRICS CLASS"):
                line = next(lines)

            # check the header
            line = next(lines)
            if header != line.strip().split("\t"):
                continue

            # one row
            line = next(lines)
            fields = [int(field) for field in line.strip("\n").split("\t")]
            sample_data = OrderedDict(zip(header, fields))
        except StopIteration:
            pass

        if sample_data:
            all_data[s_name] = OrderedDict(zip(header, fields))

    # Filter to strip out ignored sample names
    all_data = self.ignore_samples(all_data)

    if not all_data:
        return 0

    # Write parsed data to a file
    self.write_data_file(all_data, "multiqc_picard_QualityYieldMetrics")

    # Add to the general stats table
    headers = {
        "TOTAL_READS": {
            "title": "{} Reads".format(config.read_count_prefix),
            "description": "The total number of reads in the input file ({})".format(config.read_count_desc),
            "modify": lambda x: x * config.read_count_multiplier,
            "scale": "Blues",
            "shared_key": "read_count",
        }
    }
    self.general_stats_addcols(all_data, headers)

    return len(all_data)
